"""Common API helper functions to reduce code duplication"""

# Django
from django.contrib.humanize.templatetags.humanize import intcomma
from django.db.models import Sum
from django.utils import timezone
from django.utils.html import format_html
from django.utils.translation import gettext as _

# AA TaxSystem
from taxsystem.api.helpers import core
from taxsystem.api.helpers.manage import (
    generate_filter_delete_button,
    generate_ps_add_payments_button,
    generate_ps_info_button,
    generate_ps_toggle_button,
    manage_payments,
)
from taxsystem.api.helpers.statistics import (
    StatisticsResponse,
    get_members_statistics,
    get_payment_system_statistics,
    get_payments_statistics,
)
from taxsystem.api.schema import (
    AccountSchema,
    CharacterSchema,
    DashboardDivisionsSchema,
    DataTableSchema,
    DivisionSchema,
    FilterSetModelSchema,
    RequestStatusSchema,
    UpdateStatusSchema,
)
from taxsystem.helpers import lazy
from taxsystem.models.wallet import CorporationWalletJournalEntry


def get_optimized_payments_queryset(payments_model, owner, owner_id_field):
    """
    Get optimized payments queryset with select_related to prevent N+1 queries.

    Args:
        payments_model: The payments model class (CorporationPayments or AlliancePayments)
        owner: Owner object (CorporationOwner or AllianceOwner)
        owner_id_field: The owner ID value (corporation_id or alliance_id)

    Returns:
        QuerySet: Optimized payments queryset
    """
    return (
        payments_model.objects.filter(
            account__owner=owner,
            owner_id=owner_id_field,
        )
        .select_related(
            "account",
            "account__user",
            "account__user__profile",
            "account__user__profile__main_character",
        )
        .order_by("-date")
    )


def get_optimized_own_payments_queryset(payments_model, owner, account, owner_id_field):
    """
    Get optimized own payments queryset with select_related to prevent N+1 queries.

    Args:
        payments_model: The payments model class (CorporationPayments or AlliancePayments)
        owner: Owner object (CorporationOwner or AllianceOwner)
        account: Payment account object
        owner_id_field: The owner ID value (corporation_id or alliance_id)

    Returns:
        QuerySet: Optimized own payments queryset
    """
    return (
        payments_model.objects.filter(
            account__owner=owner,
            account=account,
            owner_id=owner_id_field,
        )
        .select_related(
            "account",
            "account__user",
            "account__user__profile",
            "account__user__profile__main_character",
        )
        .order_by("-date")
    )


def create_payment_response_data(payment, request, perms):
    """
    Create common payment response data

    Args:
        payment: Payment object (CorporationPayments or AlliancePayments)
        request: HTTP request object
        perms: User permissions

    Returns:
        dict: Dictionary containing payment response data
    """
    character_portrait = lazy.get_character_portrait_url(
        payment.character_id, size=32, as_html=True
    )

    # Create the action buttons
    actions_html = manage_payments(request=request, perms=perms, payment=payment)

    # Create the request status
    response_request_status = RequestStatusSchema(
        status=payment.get_request_status_display(),
        color=payment.RequestStatus(payment.request_status).color(),
    )

    return {
        "payment_id": payment.pk,
        "character": CharacterSchema(
            character_id=payment.character_id,
            character_name=payment.account.name,
            character_portrait=character_portrait,
        ),
        "amount": payment.amount,
        "date": payment.formatted_payment_date,
        "request_status": response_request_status,
        "division_name": payment.division_name,
        "reviser": payment.reviser,
        "reason": payment.reason,
        "actions": actions_html,
    }


def create_own_payment_response_data(payment):
    """
    Create payment response data for own payments view

    Args:
        payment: Payment object (CorporationPayments or AlliancePayments)

    Returns:
        dict: Dictionary containing payment response data
    """
    # Create the character portrait
    character_portrait = lazy.get_character_portrait_url(
        payment.character_id, size=32, as_html=True
    )

    # Create the actions
    actions = core.generate_info_button(payment)

    # Create the request status
    response_request_status = RequestStatusSchema(
        status=payment.get_request_status_display(),
        color=payment.RequestStatus(payment.request_status).color(),
    )

    return {
        "payment_id": payment.pk,
        "character": CharacterSchema(
            character_id=payment.character_id,
            character_name=payment.account.name,
            character_portrait=character_portrait,
        ),
        "amount": payment.amount,
        "date": payment.formatted_payment_date,
        "request_status": response_request_status,
        "division_name": payment.division_name,
        "reviser": payment.reviser,
        "reason": payment.reason,
        "actions": actions,
    }


def create_divisions_list(divisions):
    """
    Create divisions list with total balance

    Args:
        divisions: QuerySet of CorporationWalletDivision objects

    Returns:
        tuple: (list of DivisionSchema, total_balance)
    """
    response_divisions_list = []
    total_balance = 0

    for i, division in enumerate(divisions, start=1):
        division_name = division.name if division.name else f"{i}. {_('Division')}"
        response_divisions_list.append(
            DivisionSchema(
                name=division_name,
                balance=division.balance,
            )
        )
        total_balance += division.balance

    return response_divisions_list, total_balance


def create_statistics_response(owner):
    """
    Create statistics response for dashboard

    Args:
        owner: Owner object (CorporationOwner or AllianceOwner)

    Returns:
        StatisticsResponse: Statistics response object
    """
    return StatisticsResponse(
        owner_id=owner.pk,
        owner_name=owner.name,
        payment_system=get_payment_system_statistics(owner),
        payments=get_payments_statistics(owner),
        members=get_members_statistics(owner),
    )


def calculate_activity(owner, corporation_id) -> float:
    """
    Calculate activity for the past 30 days

    Args:
        owner: Owner object (CorporationOwner or AllianceOwner)
        corporation_id: Corporation ID for filtering

    Returns:
        float: Activity amount
    """
    past30_days = (
        CorporationWalletJournalEntry.objects.filter(
            division__corporation=(
                owner if hasattr(owner, "eve_corporation") else owner.corporation
            ),
            date__gte=timezone.now() - timezone.timedelta(days=30),
        )
        .exclude(first_party_id=corporation_id, second_party_id=corporation_id)
        .aggregate(total=Sum("amount"))
    )

    total_amount = past30_days.get("total", 0) or 0
    return total_amount


def create_dashboard_common_data(owner, divisions):
    """
    Create common dashboard data structure

    Args:
        owner: Owner object (CorporationOwner or AllianceOwner)
        divisions: QuerySet of CorporationWalletDivision objects

    Returns:
        dict: Dictionary containing common dashboard data
    """
    # Create divisions
    response_divisions_list, total_balance = create_divisions_list(divisions)

    # Create statistics
    response_statistics = create_statistics_response(owner)

    return {
        "update_status": UpdateStatusSchema(
            status=owner.get_update_status,
            icon=owner.get_status.bootstrap_icon(),
        ),
        "tax_amount": owner.tax_amount,
        "tax_period": owner.tax_period,
        "divisions": DashboardDivisionsSchema(
            divisions=response_divisions_list,
            total_balance=total_balance,
        ),
        "statistics": response_statistics,
    }


def create_member_response_data(member):
    """
    Create member response data

    Args:
        member: Member object

    Returns:
        dict: Dictionary containing member response data
    """
    return {
        "character": CharacterSchema(
            character_id=member.character_id,
            character_name=member.character_name,
            character_portrait=lazy.get_character_portrait_url(
                member.character_id, size=32, as_html=True
            ),
        ),
        "is_faulty": member.is_faulty,
        "status": member.get_status_display(),
        "joined": member.joined,
    }


def build_payments_response_list(payments, request, perms, payment_schema_class):
    """
    Build list of payment response objects from queryset.

    Generic function that works for both Corporation and Alliance payments.

    Args:
        payments: QuerySet of payment objects (CorporationPayments or AlliancePayments)
        request: Django request object
        perms: Permission flag
        payment_schema_class: Schema class to use for response (PaymentCorporationSchema or PaymentAllianceSchema)

    Returns:
        list: List of payment schema objects
    """
    response_payments_list = []
    for payment in payments:
        payment_data = create_payment_response_data(payment, request, perms)
        response_payment = payment_schema_class(**payment_data)
        response_payments_list.append(response_payment)
    return response_payments_list


def build_own_payments_response_list(payments, payment_schema_class):
    """
    Build list of own payment response objects from queryset.

    Generic function that works for both Corporation and Alliance own payments.

    Args:
        payments: QuerySet of payment objects (CorporationPayments or AlliancePayments)
        payment_schema_class: Schema class to use for response (PaymentCorporationSchema or PaymentAllianceSchema)

    Returns:
        list: List of payment schema objects
    """
    response_payments_list = []
    for payment in payments:
        payment_data = create_own_payment_response_data(payment)
        response_payment = payment_schema_class(**payment_data)
        response_payments_list.append(response_payment)
    return response_payments_list


def build_members_response_list(members, member_schema_class):
    """
    Build list of member response objects from queryset.

    Generic function that works for both Corporation and Alliance members.

    Args:
        members: QuerySet of Members objects
        member_schema_class: Schema class to use for response (MembersSchema)

    Returns:
        list: List of member schema objects
    """
    response_members_list = []
    for member in members:
        member_data = create_member_response_data(member)
        response_member = member_schema_class(**member_data)
        response_members_list.append(response_member)
    return response_members_list


def create_payment_account_response_data(account):
    """
    Create payment account response data for payment system.

    Args:
        account: Payment account object (CorporationPaymentAccount or AlliancePaymentAccount)

    Returns:
        dict: Dictionary containing payment account response data
    """
    character_id = account.user.profile.main_character.character_id
    character_name = account.user.profile.main_character.character_name

    # Create the action buttons
    actions = []
    actions.append(generate_ps_add_payments_button(account=account))
    actions.append(generate_ps_toggle_button(account=account))
    actions.append(generate_ps_info_button(account=account))
    actions_html = format_html(
        f'<div class="d-flex justify-content-end">{format_html("".join(actions))}</div>'
    )

    return {
        "payment_id": account.pk,
        "account": AccountSchema(
            character_id=character_id,
            character_name=character_name,
            character_portrait=lazy.get_character_portrait_url(
                character_id, size=32, as_html=True
            ),
            alt_ids=account.get_alt_ids(),
        ),
        "status": account.get_payment_status(),
        "deposit": account.deposit,
        "has_paid": DataTableSchema(
            raw=account.has_paid,
            display=account.has_paid_icon(badge=True),
            sort=str(int(account.has_paid)),
            translation=_("Has Paid"),
            dropdown_text=_("Yes") if account.has_paid else _("No"),
        ),
        "last_paid": account.last_paid,
        "next_due": account.next_due,
        "is_active": account.is_active,
        "actions": actions_html,
    }


def build_payment_accounts_response_list(payment_accounts, payment_system_schema_class):
    """
    Build list of payment account response objects from queryset.

    Generic function that works for both Corporation and Alliance payment systems.

    Args:
        payment_accounts: QuerySet of payment account objects
        payment_system_schema_class: Schema class to use for response (PaymentSystemSchema)

    Returns:
        list: List of payment system schema objects
    """
    response_payment_accounts_list = []
    for account in payment_accounts:
        account_data = create_payment_account_response_data(account)
        response_payment_account = payment_system_schema_class(**account_data)
        response_payment_accounts_list.append(response_payment_account)
    return response_payment_accounts_list


def create_filter_response_data(filter_obj):
    """
    Create filter response data.

    Args:
        filter_obj: Filter object (CorporationFilter or AllianceFilter)

    Returns:
        dict: Dictionary containing filter response data
    """
    # Format value based on filter type
    if filter_obj.filter_type == filter_obj.__class__.FilterType.AMOUNT:
        value = f"{intcomma(filter_obj.value, use_l10n=True)} ISK"
    else:
        value = filter_obj.value

    # Create actions
    actions = []
    actions.append(generate_filter_delete_button(filter_obj=filter_obj))
    actions_html = format_html(
        f'<div class="d-flex justify-content-end">{format_html("".join(actions))}</div>'
    )

    return {
        "filter_set": FilterSetModelSchema(
            owner_id=filter_obj.filter_set.owner.pk,
            name=filter_obj.filter_set.name,
            description=filter_obj.filter_set.description,
            enabled=filter_obj.filter_set.enabled,
        ),
        "filter_type": filter_obj.get_filter_type_display(),
        "value": value,
        "actions": actions_html,
    }


def build_filters_response_list(filters, filter_schema_class):
    """
    Build list of filter response objects from queryset.

    Generic function that works for both Corporation and Alliance filters.

    Args:
        filters: QuerySet of filter objects
        filter_schema_class: Schema class to use for response (FilterModelSchema)

    Returns:
        list: List of filter schema objects
    """
    response_filters_list = []
    for filter_obj in filters:
        filter_data = create_filter_response_data(filter_obj)
        response_filter = filter_schema_class(**filter_data)
        response_filters_list.append(response_filter)
    return response_filters_list


def create_admin_log_response_data(log):
    """
    Create admin log response data.

    Args:
        log: Admin history object (CorporationAdminHistory or AllianceAdminHistory)

    Returns:
        dict: Dictionary containing admin log response data
    """
    return {
        "log_id": log.pk,
        "user_name": log.user.username,
        "date": timezone.localtime(log.date).strftime("%Y-%m-%d %H:%M"),
        "action": log.action,
        "comment": log.comment,
    }


def build_admin_logs_response_list(logs, admin_log_schema_class):
    """
    Build list of admin log response objects from queryset.

    Generic function that works for both Corporation and Alliance admin logs.

    Args:
        logs: QuerySet of admin history objects
        admin_log_schema_class: Schema class to use for response (AdminHistorySchema)

    Returns:
        list: List of admin history schema objects
    """
    response_logs_list = []
    for log in logs:
        log_data = create_admin_log_response_data(log)
        response_log = admin_log_schema_class(**log_data)
        response_logs_list.append(response_log)
    return response_logs_list



# Django
from django.test import RequestFactory, TestCase
from django.urls import reverse

# AA TaxSystem
from taxsystem.models.alliance import AllianceOwner
from taxsystem.models.corporation import CorporationOwner
from taxsystem.tests.testdata.generate_owneraudit import (
    create_alliance_owner_from_user,
    create_corporation_owner_from_user,
    create_user_from_evecharacter_with_access,
)
from taxsystem.tests.testdata.load_allianceauth import load_allianceauth
from taxsystem.tests.testdata.load_eveuniverse import load_eveuniverse
from taxsystem.views import owner_overview


class TestOwnerOverview(TestCase):
    """Test owner_overview view function."""

    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        load_allianceauth()
        load_eveuniverse()

        cls.factory = RequestFactory()
        # Create user with main character (has basic_access permission)
        cls.user, cls.character_ownership = create_user_from_evecharacter_with_access(
            1001
        )

        # Create corporation owner (visible via basic_access)
        cls.corp_owner = create_corporation_owner_from_user(cls.user)

        # Create alliance owner (visible via basic_access)
        cls.alliance_owner = create_alliance_owner_from_user(cls.user)

    def test_owner_overview_displays_corporations(self):
        """Test that owner overview displays corporations for user with access"""
        # Arrange
        request = self.factory.get(reverse("taxsystem:owner_overview"))
        request.user = self.user

        # Act
        response = owner_overview(request)

        # Assert
        self.assertEqual(response.status_code, 200)
        self.assertIn(b"Owner Overview", response.content)
        self.assertIn(
            self.corp_owner.eve_corporation.corporation_name.encode(), response.content
        )

    def test_owner_overview_displays_alliances(self):
        """Test that owner overview displays alliances for user with access"""
        # Arrange
        request = self.factory.get(reverse("taxsystem:owner_overview"))
        request.user = self.user

        # Act
        response = owner_overview(request)

        # Assert
        self.assertEqual(response.status_code, 200)
        self.assertIn(
            self.alliance_owner.eve_alliance.alliance_name.encode(), response.content
        )

    def test_owner_overview_counts_owners_correctly(self):
        """Test that owner overview counts corporations and alliances correctly"""
        # Arrange
        self.client.force_login(self.user)

        # Act
        response = self.client.get(reverse("taxsystem:owner_overview"))

        # Assert
        self.assertEqual(response.status_code, 200)
        # Should have at least 1 corporation and 1 alliance
        self.assertIn(b"Corporation", response.content)
        self.assertIn(b"Alliance", response.content)

    def test_owner_overview_includes_portraits(self):
        """Test that owner overview includes corporation and alliance logos"""
        # Arrange
        request = self.factory.get(reverse("taxsystem:owner_overview"))
        request.user = self.user

        # Act
        response = owner_overview(request)

        # Assert
        self.assertEqual(response.status_code, 200)
        # Check for image server URLs
        self.assertIn(b"https://images.evetech.net", response.content)
        self.assertIn(b"corporation", response.content)
        self.assertIn(b"alliance", response.content)

    def test_owner_overview_includes_action_buttons(self):
        """Test that owner overview includes action buttons for each owner"""
        # Arrange
        request = self.factory.get(reverse("taxsystem:owner_overview"))
        request.user = self.user

        # Act
        response = owner_overview(request)

        # Assert
        self.assertEqual(response.status_code, 200)
        # Check for Payments and Manage buttons
        self.assertIn(b"Payments", response.content)
        self.assertIn(b"Manage", response.content)

    def test_owner_overview_with_no_owners(self):
        """Test that owner overview shows empty message when user has no access"""
        # Arrange
        user_no_access, _ = create_user_from_evecharacter_with_access(1002)
        request = self.factory.get(reverse("taxsystem:owner_overview"))
        request.user = user_no_access

        # Act
        response = owner_overview(request)

        # Assert
        self.assertEqual(response.status_code, 200)
        self.assertIn(b"No owners found", response.content)

    def test_owner_overview_sorts_by_name(self):
        """Test that owners are sorted by name"""
        # Arrange
        request = self.factory.get(reverse("taxsystem:owner_overview"))
        request.user = self.user

        # Act
        response = owner_overview(request)

        # Assert
        self.assertEqual(response.status_code, 200)
        # Response should contain owners
        content = response.content.decode("utf-8")
        self.assertIn("Owner Overview", content)

    def test_owner_overview_uses_select_related(self):
        """Test that owner overview uses select_related for performance"""
        # Arrange
        request = self.factory.get(reverse("taxsystem:owner_overview"))
        request.user = self.user

        # Act - Note: get_status property makes additional queries for each owner
        # so we just verify the view works, not the exact query count
        response = owner_overview(request)

        # Assert
        self.assertEqual(response.status_code, 200)
        # Verify select_related is used by checking response has owners
        content = response.content.decode("utf-8")
        self.assertIn(self.corp_owner.eve_corporation.corporation_name, content)
        self.assertIn(self.alliance_owner.eve_alliance.alliance_name, content)

    def test_owner_overview_shows_inactive_owners(self):
        """Test that inactive owners are displayed with appropriate styling"""
        # Arrange
        self.corp_owner.active = False
        self.corp_owner.save()

        request = self.factory.get(reverse("taxsystem:owner_overview"))
        request.user = self.user

        # Act
        response = owner_overview(request)

        # Assert
        self.assertEqual(response.status_code, 200)
        self.assertIn(b"Inactive", response.content)

        # Cleanup
        self.corp_owner.active = True
        self.corp_owner.save()

    def test_index_redirects_to_owner_overview(self):
        """Test that index redirects to owner overview"""
        # Arrange
        self.client.force_login(self.user)
        url = reverse("taxsystem:index")

        # Act
        response = self.client.get(url, follow=False)

        # Assert
        self.assertEqual(response.status_code, 302)
        self.assertEqual(response.url, reverse("taxsystem:owner_overview"))



"""Models for Tax System."""

# Django
from django.db import models
from django.utils.translation import gettext_lazy as _

# Alliance Auth
from allianceauth.eveonline.models import (
    EveAllianceInfo,
)
from allianceauth.services.hooks import get_extension_logger

# Alliance Auth (External Libs)
from app_utils.logging import LoggerAddTag

# AA TaxSystem
from taxsystem import __title__
from taxsystem.managers.alliance_manager import (
    AllianceOwnerManager,
    AlliancePaymentAccountManager,
    AlliancePaymentManager,
)
from taxsystem.models.base import (
    AdminHistoryBase,
    FilterBase,
    FilterSetBase,
    OwnerBase,
    PaymentAccountBase,
    PaymentHistoryBase,
    PaymentsBase,
    UpdateStatusBase,
)
from taxsystem.models.corporation import CorporationOwner
from taxsystem.models.general import AllianceUpdateSection
from taxsystem.models.wallet import CorporationWalletJournalEntry

logger = LoggerAddTag(get_extension_logger(__name__), __title__)


class AllianceUpdateStatus(UpdateStatusBase):
    """Model representing the update status of an alliance owner in the tax system."""

    owner = models.ForeignKey(
        "AllianceOwner",
        on_delete=models.CASCADE,
        related_name="ts_alliance_update_status",
    )

    section = models.CharField(
        max_length=32, choices=AllianceUpdateSection.choices, db_index=True
    )

    class Meta:
        default_permissions = ()
        unique_together = [("owner", "section")]

    def __str__(self) -> str:
        return f"{self.owner.name} - {self.section}"


class AllianceOwner(OwnerBase):
    """Model representing an alliance owner in the tax system."""

    class Meta:
        default_permissions = ()

    objects: AllianceOwnerManager = AllianceOwnerManager()

    eve_alliance = models.OneToOneField(
        EveAllianceInfo,
        on_delete=models.CASCADE,
        related_name="+",
    )

    corporation = models.ForeignKey(
        CorporationOwner,
        on_delete=models.PROTECT,
        related_name="+",
        help_text=_("The corporation that owns this alliance tax system."),
    )

    def __str__(self) -> str:
        return f"{self.eve_alliance.alliance_name}"

    # Abstract properties implementation
    @property
    def eve_id(self) -> int:
        """Return the Eve Alliance ID."""
        return self.eve_alliance.alliance_id

    @property
    def payment_accounts_manager(self):
        """Return the alliance payment accounts related manager."""
        return self.ts_alliance_payment_accounts

    @property
    def update_status_manager(self) -> models.QuerySet[AllianceUpdateStatus]:
        """Return the related manager for alliance update status objects."""
        return AllianceUpdateStatus.objects.filter(owner=self)

    @property
    def update_section_enum(self):
        """Return the alliance update section enum class."""
        return AllianceUpdateSection

    @property
    def payments_class(self) -> type["AlliancePayments"]:
        """Return the payments class for this alliance owner."""
        return AlliancePayments

    @property
    def payments_account_class(self) -> type["AlliancePaymentAccount"]:
        """Return the payments account class for this alliance owner."""
        return AlliancePaymentAccount

    @property
    def payments_history_class(self) -> type["AlliancePaymentHistory"]:
        """Return the payments class for this alliance owner."""
        return AlliancePaymentHistory

    @property
    def admin_history_class(self) -> type["AllianceAdminHistory"]:
        """Return the admin history class for this alliance owner."""
        return AllianceAdminHistory

    @property
    def filterset_class(self) -> type["AllianceFilterSet"]:
        """Return the filter set class for this alliance owner."""
        return AllianceFilterSet

    @property
    def filter_class(self) -> type["AllianceFilter"]:
        """Return the filter class for this alliance owner."""
        return AllianceFilter


class AlliancePaymentAccount(PaymentAccountBase):
    """Model representing an alliance payment account in the tax system."""

    objects: AlliancePaymentAccountManager = AlliancePaymentAccountManager()

    class Meta:
        default_permissions = ()

    owner = models.ForeignKey(
        AllianceOwner,
        on_delete=models.CASCADE,
        related_name="ts_alliance_payment_accounts",
    )

    def __str__(self) -> str:
        return f"{self.name}"


class AlliancePayments(PaymentsBase):
    """Model representing payments made by alliance members in the tax system."""

    objects: AlliancePaymentManager = AlliancePaymentManager()

    class Meta:
        default_permissions = ()
        indexes = [
            models.Index(fields=["account", "owner_id", "request_status", "-date"]),
            models.Index(fields=["request_status", "-date"]),
        ]

    account = models.ForeignKey(
        AlliancePaymentAccount,
        on_delete=models.CASCADE,
        related_name="ts_alliance_payments",
    )

    def __str__(self) -> str:
        return f"{self.account.name} - {self.amount} ISK"


class AllianceFilterSet(FilterSetBase):
    owner = models.ForeignKey(
        AllianceOwner,
        on_delete=models.CASCADE,
        related_name="ts_alliance_filter_set",
    )

    def filter(self, payments: AlliancePayments) -> models.QuerySet[AlliancePayments]:
        if self.is_active:
            for f in self.ts_alliance_filters.all():
                payments = f.apply_filter(payments)
            return payments
        return AlliancePayments.objects.none()

    def filter_contains(
        self, payments: AlliancePayments
    ) -> models.QuerySet[AlliancePayments]:  # not implemented yet
        if self.is_active:
            for f in self.ts_alliance_filters.all():
                payments = f.apply_contains(payments)
            return payments
        return AlliancePayments.objects.none()


class AllianceFilter(FilterBase):
    filter_set = models.ForeignKey(
        AllianceFilterSet,
        on_delete=models.CASCADE,
        related_name="ts_alliance_filters",
    )

    def apply_filter(
        self, qs: models.QuerySet[CorporationWalletJournalEntry]
    ) -> models.QuerySet[CorporationWalletJournalEntry]:
        if self.filter_type == AllianceFilter.FilterType.REASON:
            return qs.filter(reason=self.value)
        if self.filter_type == AllianceFilter.FilterType.AMOUNT:
            return qs.filter(amount=self.value)
        # weitere Felder
        return qs

    def apply_contains(
        self, qs: models.QuerySet[CorporationWalletJournalEntry]
    ) -> models.QuerySet[CorporationWalletJournalEntry]:
        if self.filter_type == AllianceFilter.FilterType.REASON:
            return qs.filter(reason__icontains=self.value)
        if self.filter_type == AllianceFilter.FilterType.AMOUNT:
            return qs.filter(amount__gte=self.value)
        # weitere Felder
        return qs

    class Meta:
        default_permissions = ()


class AlliancePaymentHistory(PaymentHistoryBase):
    """Model representing the history of actions taken on alliance payments in the tax system."""

    class Meta:
        default_permissions = ()

    # pylint: disable=duplicate-code
    payment = models.ForeignKey(
        AlliancePayments,
        on_delete=models.CASCADE,
        related_name="+",
        verbose_name=_("Payment"),
        help_text=_("Payment that the action was performed on"),
    )
    # pylint: enable=duplicate-code
    new_status = models.CharField(
        max_length=16,
        choices=AlliancePayments.RequestStatus.choices,
        verbose_name=_("New Status"),
        help_text=_("New Status of the action"),
    )


class AllianceAdminHistory(AdminHistoryBase):
    """Model representing the history of administrative actions taken on alliance owners in the tax system."""

    class Meta:
        default_permissions = ()

    owner = models.ForeignKey(
        AllianceOwner,
        on_delete=models.CASCADE,
        related_name="+",
        verbose_name=_("Payment"),
        help_text=_("Payment that the action was performed on"),
    )



# Django
from django.test import RequestFactory, TestCase
from django.utils import timezone

# AA TaxSystem
from taxsystem.api.corporation import PaymentCorporationSchema
from taxsystem.api.helpers.common import (
    build_admin_logs_response_list,
    build_filters_response_list,
    build_members_response_list,
    build_own_payments_response_list,
    build_payment_accounts_response_list,
    build_payments_response_list,
    create_admin_log_response_data,
    create_filter_response_data,
    create_payment_account_response_data,
)
from taxsystem.api.schema import (
    AdminHistorySchema,
    FilterModelSchema,
    MembersSchema,
    PaymentSystemSchema,
)
from taxsystem.models.corporation import (
    CorporationAdminHistory,
    CorporationFilter,
    CorporationFilterSet,
    CorporationOwner,
    CorporationPaymentAccount,
    CorporationPayments,
    Members,
)
from taxsystem.tests.testdata.generate_owneraudit import (
    create_corporation_owner_from_user,
    create_user_from_evecharacter_with_access,
)
from taxsystem.tests.testdata.generate_payments import (
    create_payment,
    create_payment_system,
)
from taxsystem.tests.testdata.load_allianceauth import load_allianceauth
from taxsystem.tests.testdata.load_eveuniverse import load_eveuniverse


class TestBuildPaymentsResponseList(TestCase):
    """
    Test build_payments_response_list helper function.

    Prerequisites:
    - load_allianceauth() and load_eveuniverse() must be called first
    - Test user must be created via generate_owneraudit functions
    - At least one owneraudit (CorporationOwner) must exist
    """

    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        # Load base test data (required for all tests)
        load_allianceauth()
        load_eveuniverse()

        cls.factory = RequestFactory()
        # Create user with main character using testdata factory
        cls.user, cls.character_ownership = create_user_from_evecharacter_with_access(
            1001
        )

        # Create owneraudit (CorporationOwner) from user
        cls.owner = create_corporation_owner_from_user(cls.user)

    def test_builds_corporation_payment_list(self):
        """Test building corporation payment response list"""
        # Arrange
        account = create_payment_system(self.owner, user=self.user)
        payment1 = create_payment(
            account,
            name=self.user.profile.main_character.character_name,
            owner_id=self.owner.eve_corporation.corporation_id,
            entry_id=1,
            amount=1000000,
            date=timezone.now(),
            reason="Test Payment 1",
        )
        payment2 = create_payment(
            account,
            name=self.user.profile.main_character.character_name,
            owner_id=self.owner.eve_corporation.corporation_id,
            entry_id=2,
            amount=2000000,
            date=timezone.now(),
            reason="Test Payment 2",
        )

        payments = CorporationPayments.objects.filter(
            owner_id=self.owner.eve_corporation.corporation_id
        )

        request = self.factory.get("/")
        request.user = self.user
        perms = True

        # Act
        result = build_payments_response_list(
            payments, request, perms, PaymentCorporationSchema
        )

        # Assert
        self.assertEqual(len(result), 2)
        self.assertIsInstance(result[0], PaymentCorporationSchema)
        self.assertIsInstance(result[1], PaymentCorporationSchema)
        self.assertIn(payment1.id, [r.payment_id for r in result])
        self.assertIn(payment2.id, [r.payment_id for r in result])

    def test_builds_empty_list_for_no_payments(self):
        """Test building empty list when no payments exist"""
        # Arrange
        payments = CorporationPayments.objects.none()
        request = self.factory.get("/")
        request.user = self.user
        perms = True

        # Act
        result = build_payments_response_list(
            payments, request, perms, PaymentCorporationSchema
        )

        # Assert
        self.assertEqual(len(result), 0)
        self.assertIsInstance(result, list)


class TestBuildOwnPaymentsResponseList(TestCase):
    """
    Test build_own_payments_response_list helper function.

    Prerequisites:
    - load_allianceauth() and load_eveuniverse() must be called first
    - Test user must be created via generate_owneraudit functions
    - At least one owneraudit (CorporationOwner) must exist
    """

    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        # Load base test data (required for all tests)
        load_allianceauth()
        load_eveuniverse()

        # Create user with main character using testdata factory
        cls.user, cls.character_ownership = create_user_from_evecharacter_with_access(
            1001
        )

        # Create owneraudit (CorporationOwner) from user
        cls.owner = create_corporation_owner_from_user(cls.user)

    def test_builds_own_corporation_payment_list(self):
        """Test building own payment response list for corporation"""
        # Arrange
        account = create_payment_system(self.owner, user=self.user)
        create_payment(
            account,
            name=self.user.profile.main_character.character_name,
            owner_id=self.owner.eve_corporation.corporation_id,
            entry_id=1,
            amount=500000,
            date=timezone.now(),
            reason="Test Payment 1",
        )
        create_payment(
            account,
            name=self.user.profile.main_character.character_name,
            owner_id=self.owner.eve_corporation.corporation_id,
            entry_id=2,
            amount=750000,
            date=timezone.now(),
            reason="Test Payment 2",
        )

        payments = CorporationPayments.objects.filter(
            owner_id=self.owner.eve_corporation.corporation_id,
            account__user=self.user,
        )

        # Act
        result = build_own_payments_response_list(payments, PaymentCorporationSchema)

        # Assert
        self.assertEqual(len(result), 2)
        self.assertIsInstance(result[0], PaymentCorporationSchema)
        self.assertIsInstance(result[1], PaymentCorporationSchema)


class TestBuildMembersResponseList(TestCase):
    """
    Test build_members_response_list helper function.

    Prerequisites:
    - load_allianceauth() and load_eveuniverse() must be called first
    - Test user must be created via generate_owneraudit functions
    - At least one owneraudit (CorporationOwner) must exist
    """

    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        # Load base test data (required for all tests)
        load_allianceauth()
        load_eveuniverse()

        # Create user with main character using testdata factory
        cls.user, cls.character_ownership = create_user_from_evecharacter_with_access(
            1001
        )

        # Create owneraudit (CorporationOwner) from user
        cls.owner = create_corporation_owner_from_user(cls.user)

    def test_builds_members_list(self):
        """Test building members response list with actual member data"""
        # Arrange - Create test member
        Members.objects.create(
            owner=self.owner,
            character_id=self.user.profile.main_character.character_id,
            character_name=self.user.profile.main_character.character_name,
            joined=timezone.now(),
        )

        members = Members.objects.filter(
            owner__eve_corporation__corporation_id=self.owner.eve_corporation.corporation_id
        ).select_related("owner", "owner__eve_corporation")

        # Act
        result = build_members_response_list(members, MembersSchema)

        # Assert
        self.assertGreater(len(result), 0)
        for member_response in result:
            self.assertIsInstance(member_response, MembersSchema)
            self.assertIsNotNone(member_response.character)

    def test_builds_empty_members_list(self):
        """Test building empty members list"""
        # Arrange
        members = Members.objects.none()

        # Act
        result = build_members_response_list(members, MembersSchema)

        # Assert
        self.assertEqual(len(result), 0)
        self.assertIsInstance(result, list)


class TestCreatePaymentAccountResponseData(TestCase):
    """Test create_payment_account_response_data helper function."""

    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        load_allianceauth()
        load_eveuniverse()

        cls.user, cls.character_ownership = create_user_from_evecharacter_with_access(
            1001
        )
        cls.owner = create_corporation_owner_from_user(cls.user)

    def test_creates_payment_account_data(self):
        """Test creating payment account response data"""
        # Arrange
        account = create_payment_system(self.owner, user=self.user)

        # Act
        result = create_payment_account_response_data(account)

        # Assert
        self.assertIsInstance(result, dict)
        self.assertIn("payment_id", result)
        self.assertIn("account", result)
        self.assertIn("has_paid", result)
        self.assertIn("actions", result)
        self.assertEqual(result["payment_id"], account.pk)

    def test_payment_account_includes_character_info(self):
        """Test that payment account data includes character information"""
        # Arrange
        account = create_payment_system(self.owner, user=self.user)

        # Act
        result = create_payment_account_response_data(account)

        # Assert
        account_schema = result["account"]
        self.assertEqual(
            account_schema.character_id,
            self.user.profile.main_character.character_id,
        )
        self.assertEqual(
            account_schema.character_name,
            self.user.profile.main_character.character_name,
        )


class TestBuildPaymentAccountsResponseList(TestCase):
    """Test build_payment_accounts_response_list helper function."""

    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        load_allianceauth()
        load_eveuniverse()

        cls.user, cls.character_ownership = create_user_from_evecharacter_with_access(
            1001
        )
        cls.owner = create_corporation_owner_from_user(cls.user)

    def test_builds_payment_accounts_list(self):
        """Test building payment accounts response list"""
        # Arrange
        create_payment_system(self.owner, user=self.user)

        # Create second user and account
        user2, _ = create_user_from_evecharacter_with_access(1002)
        create_payment_system(self.owner, user=user2)

        accounts = CorporationPaymentAccount.objects.filter(
            owner=self.owner
        ).select_related(
            "user",
            "user__profile",
            "user__profile__main_character",
        )

        # Act
        result = build_payment_accounts_response_list(accounts, PaymentSystemSchema)

        # Assert
        self.assertEqual(len(result), 2)
        self.assertIsInstance(result[0], PaymentSystemSchema)
        self.assertIsInstance(result[1], PaymentSystemSchema)

    def test_builds_empty_payment_accounts_list(self):
        """Test building empty payment accounts list"""
        # Arrange
        accounts = CorporationPaymentAccount.objects.none()

        # Act
        result = build_payment_accounts_response_list(accounts, PaymentSystemSchema)

        # Assert
        self.assertEqual(len(result), 0)
        self.assertIsInstance(result, list)


class TestCreateFilterResponseData(TestCase):
    """Test create_filter_response_data helper function."""

    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        load_allianceauth()
        load_eveuniverse()

        cls.user, cls.character_ownership = create_user_from_evecharacter_with_access(
            1001
        )
        cls.owner = create_corporation_owner_from_user(cls.user)

    def test_creates_filter_data_with_amount_formatting(self):
        """Test creating filter response data with AMOUNT type formatting"""
        # Arrange
        filter_set = CorporationFilterSet.objects.create(
            owner=self.owner,
            name="Test Filter Set",
            description="Test Description",
            enabled=True,
        )
        filter_obj = CorporationFilter.objects.create(
            filter_set=filter_set,
            filter_type=CorporationFilter.FilterType.AMOUNT,
            value=1000000,
        )

        # Act
        result = create_filter_response_data(filter_obj)

        # Assert
        self.assertIsInstance(result, dict)
        self.assertIn("filter_set", result)
        self.assertIn("filter_type", result)
        self.assertIn("value", result)
        self.assertIn("actions", result)
        self.assertIn("ISK", result["value"])  # Check ISK formatting

    def test_creates_filter_data_without_amount_formatting(self):
        """Test creating filter response data with non-AMOUNT type"""
        # Arrange
        filter_set = CorporationFilterSet.objects.create(
            owner=self.owner,
            name="Test Filter Set",
            description="Test Description",
            enabled=True,
        )
        filter_obj = CorporationFilter.objects.create(
            filter_set=filter_set,
            filter_type=CorporationFilter.FilterType.REASON,
            value="Test Reason",
        )

        # Act
        result = create_filter_response_data(filter_obj)

        # Assert
        self.assertIsInstance(result, dict)
        self.assertEqual(result["value"], "Test Reason")
        self.assertNotIn("ISK", result["value"])


class TestBuildFiltersResponseList(TestCase):
    """Test build_filters_response_list helper function."""

    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        load_allianceauth()
        load_eveuniverse()

        cls.user, cls.character_ownership = create_user_from_evecharacter_with_access(
            1001
        )
        cls.owner = create_corporation_owner_from_user(cls.user)

    def test_builds_filters_list(self):
        """Test building filters response list"""
        # Arrange
        filter_set = CorporationFilterSet.objects.create(
            owner=self.owner,
            name="Test Filter Set",
            description="Test Description",
            enabled=True,
        )
        CorporationFilter.objects.create(
            filter_set=filter_set,
            filter_type=CorporationFilter.FilterType.AMOUNT,
            value=1000000,
        )
        CorporationFilter.objects.create(
            filter_set=filter_set,
            filter_type=CorporationFilter.FilterType.REASON,
            value="Test Reason",
        )

        filters = CorporationFilter.objects.filter(
            filter_set=filter_set
        ).select_related("filter_set", "filter_set__owner")

        # Act
        result = build_filters_response_list(filters, FilterModelSchema)

        # Assert
        self.assertEqual(len(result), 2)
        self.assertIsInstance(result[0], FilterModelSchema)
        self.assertIsInstance(result[1], FilterModelSchema)

    def test_builds_empty_filters_list(self):
        """Test building empty filters list"""
        # Arrange
        filters = CorporationFilter.objects.none()

        # Act
        result = build_filters_response_list(filters, FilterModelSchema)

        # Assert
        self.assertEqual(len(result), 0)
        self.assertIsInstance(result, list)


class TestCreateAdminLogResponseData(TestCase):
    """Test create_admin_log_response_data helper function."""

    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        load_allianceauth()
        load_eveuniverse()

        cls.user, cls.character_ownership = create_user_from_evecharacter_with_access(
            1001
        )
        cls.owner = create_corporation_owner_from_user(cls.user)

    def test_creates_admin_log_data(self):
        """Test creating admin log response data"""
        # Arrange
        log = CorporationAdminHistory.objects.create(
            owner=self.owner,
            user=self.user,
            action=CorporationAdminHistory.Actions.ADD,
            comment="Test log entry",
        )

        # Act
        result = create_admin_log_response_data(log)

        # Assert
        self.assertIsInstance(result, dict)
        self.assertIn("log_id", result)
        self.assertIn("user_name", result)
        self.assertIn("date", result)
        self.assertIn("action", result)
        self.assertIn("comment", result)
        self.assertEqual(result["log_id"], log.pk)
        self.assertEqual(result["user_name"], self.user.username)
        self.assertEqual(result["action"], CorporationAdminHistory.Actions.ADD)
        self.assertEqual(result["comment"], "Test log entry")

    def test_admin_log_date_formatting(self):
        """Test that admin log date is properly formatted"""
        # Arrange
        log = CorporationAdminHistory.objects.create(
            owner=self.owner,
            user=self.user,
            action=CorporationAdminHistory.Actions.CHANGE,
            comment="Date format test",
        )

        # Act
        result = create_admin_log_response_data(log)

        # Assert
        self.assertIsInstance(result["date"], str)
        self.assertRegex(result["date"], r"\d{4}-\d{2}-\d{2} \d{2}:\d{2}")


class TestBuildAdminLogsResponseList(TestCase):
    """Test build_admin_logs_response_list helper function."""

    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        load_allianceauth()
        load_eveuniverse()

        cls.user, cls.character_ownership = create_user_from_evecharacter_with_access(
            1001
        )
        cls.owner = create_corporation_owner_from_user(cls.user)

    def test_builds_admin_logs_list(self):
        """Test building admin logs response list"""
        # Arrange
        CorporationAdminHistory.objects.create(
            owner=self.owner,
            user=self.user,
            action=CorporationAdminHistory.Actions.ADD,
            comment="First log entry",
        )
        CorporationAdminHistory.objects.create(
            owner=self.owner,
            user=self.user,
            action=CorporationAdminHistory.Actions.CHANGE,
            comment="Second log entry",
        )

        logs = CorporationAdminHistory.objects.filter(owner=self.owner).select_related(
            "user"
        )

        # Act
        result = build_admin_logs_response_list(logs, AdminHistorySchema)

        # Assert
        self.assertEqual(len(result), 2)
        self.assertIsInstance(result[0], AdminHistorySchema)
        self.assertIsInstance(result[1], AdminHistorySchema)

    def test_builds_empty_admin_logs_list(self):
        """Test building empty admin logs list"""
        # Arrange
        logs = CorporationAdminHistory.objects.none()

        # Act
        result = build_admin_logs_response_list(logs, AdminHistorySchema)

        # Assert
        self.assertEqual(len(result), 0)
        self.assertIsInstance(result, list)



"""
Constants
"""

# Standard Library
import os

AA_TAXSYSTEM_BASE_DIR = os.path.join(os.path.dirname(__file__))
AA_TAXSYSTEM_STATIC_DIR = os.path.join(AA_TAXSYSTEM_BASE_DIR, "static", "taxsystem")

# Common select_related fields for UserProfile / main_character joins
AUTH_SELECT_RELATED_MAIN_CHARACTER = (
    "user__profile__main_character",
    "main_character__character_ownership",
    "main_character__character_ownership__user__profile",
    "main_character__character_ownership__user__profile__main_character",
)



"""PvE Views"""

# Django
from django.contrib import messages
from django.contrib.auth.decorators import login_required, permission_required
from django.contrib.humanize.templatetags.humanize import intcomma
from django.core.exceptions import ValidationError
from django.core.handlers.wsgi import WSGIRequest
from django.db import IntegrityError, transaction
from django.http import JsonResponse
from django.shortcuts import get_object_or_404, redirect, render
from django.urls import reverse
from django.utils import timezone
from django.utils.translation import gettext_lazy as _
from django.views.decorators.csrf import csrf_exempt
from django.views.decorators.http import require_POST

# Alliance Auth
from allianceauth.authentication.decorators import permissions_required
from allianceauth.authentication.models import UserProfile
from allianceauth.eveonline.models import (
    EveAllianceInfo,
    EveCharacter,
    EveCorporationInfo,
)
from allianceauth.services.hooks import get_extension_logger
from esi.decorators import token_required

# Alliance Auth (External Libs)
from app_utils.logging import LoggerAddTag

# AA TaxSystem
from taxsystem import __title__, forms, tasks
from taxsystem.api.helpers.core import (
    get_alliance,
    get_character_permissions,
    get_corporation,
    get_manage_corporation,
    get_manage_owner,
    get_owner,
)
from taxsystem.helpers import lazy
from taxsystem.helpers.views import add_info_to_context
from taxsystem.helpers.views_generic import (
    get_default_owner_id,
    get_owner_context_key,
    get_owner_display_name,
    get_owner_template,
    get_owner_type_from_instance,
)
from taxsystem.models.alliance import (
    AllianceAdminHistory,
    AllianceOwner,
)
from taxsystem.models.corporation import (
    CorporationAdminHistory,
    CorporationOwner,
    Members,
)

logger = LoggerAddTag(get_extension_logger(__name__), __title__)


@login_required
@permission_required("taxsystem.basic_access")
def admin(request: WSGIRequest):
    corporation_id = request.user.profile.main_character.corporation_id
    if not request.user.is_superuser:
        messages.error(request, _("You do not have permission to access this page."))
        return redirect("taxsystem:index")

    def _handle_taxsystem_updates(force_refresh):
        messages.info(request, _("Queued Update All Taxsystem"))
        tasks.update_all_taxsytem.apply_async(
            kwargs={"force_refresh": force_refresh}, priority=7
        )

    def _handle_corporation_updates(force_refresh):
        corporation_id_input = request.POST.get("corporation_id")
        if corporation_id_input:
            try:
                corp_id = int(corporation_id_input)
                corporation = CorporationOwner.objects.get(
                    eve_corporation__corporation_id=corp_id
                )
                messages.info(
                    request,
                    _("Queued Update for Corporation: %s") % corporation.name,
                )
                tasks.update_corporation.apply_async(
                    args=[corporation.pk],
                    kwargs={"force_refresh": force_refresh},
                    priority=7,
                )
            except (ValueError, CorporationOwner.DoesNotExist):
                messages.error(
                    request,
                    _("Corporation with ID %s not found") % corporation_id_input,
                )
        else:
            messages.info(request, _("Queued Update All Taxsystem Corporations"))
            corporations = CorporationOwner.objects.filter(active=True)
            for corporation in corporations:
                tasks.update_corporation.apply_async(
                    args=[corporation.pk],
                    kwargs={"force_refresh": force_refresh},
                    priority=7,
                )

    def _handle_alliance_updates(force_refresh):
        alliance_id_input = request.POST.get("alliance_id")
        if alliance_id_input:
            try:
                ally_id = int(alliance_id_input)
                alliance = AllianceOwner.objects.get(eve_alliance__alliance_id=ally_id)
                messages.info(
                    request, _("Queued Update for Alliance: %s") % alliance.name
                )
                tasks.update_alliance.apply_async(
                    args=[alliance.pk],
                    kwargs={"force_refresh": force_refresh},
                    priority=7,
                )
            except (ValueError, AllianceOwner.DoesNotExist):
                messages.error(
                    request, _("Alliance with ID %s not found") % alliance_id_input
                )
        else:
            messages.info(request, _("Queued Update All Taxsystem Alliances"))
            alliances = AllianceOwner.objects.filter(active=True)
            for alliance in alliances:
                tasks.update_alliance.apply_async(
                    args=[alliance.pk],
                    kwargs={"force_refresh": force_refresh},
                    priority=7,
                )

    if request.method == "POST":
        force_refresh = bool(request.POST.get("force_refresh", False))
        if request.POST.get("run_taxsystem_updates"):
            _handle_taxsystem_updates(force_refresh)
        if request.POST.get("run_taxsystem_corporation_updates"):
            _handle_corporation_updates(force_refresh)
        if request.POST.get("run_taxsystem_alliance_updates"):
            _handle_alliance_updates(force_refresh)

    context = {
        "corporation_id": corporation_id,
        "title": _("Tax System Superuser Administration"),
    }
    return render(request, "taxsystem/admin.html", context=context)


@login_required
@permission_required("taxsystem.basic_access")
def index(request: WSGIRequest):  # pylint: disable=unused-argument
    """Index View - Redirects to Owner Overview"""
    return redirect("taxsystem:owner_overview")


@login_required
@permission_required("taxsystem.basic_access")
def owner_overview(request: WSGIRequest):
    """
    Owner Overview - Unified view for all Corporations and Alliances.

    Shows all owners (corporations and alliances) that the user has access to
    in a single DataTable with portraits, names, types, and action buttons.

    Args:
        request: The HTTP request object containing user information

    Returns:
        HttpResponse: Rendered owner overview template with combined owner list
    """
    owners = []

    # Get visible corporations using manager method
    corps = CorporationOwner.objects.visible_to(request.user).select_related(
        "eve_corporation"
    )

    for corp in corps:
        owners.append(
            {
                "type": "corporation",
                "type_display": _("Corporation"),
                "id": corp.eve_corporation.corporation_id,
                "name": corp.eve_corporation.corporation_name,
                "portrait": lazy.get_corporation_logo_url(
                    corp.eve_corporation.corporation_id,
                    size=64,
                    corporation_name=corp.eve_corporation.corporation_name,
                    as_html=True,
                ),
                "active": corp.active,
            }
        )

    # Get visible alliances using manager method
    alliances = AllianceOwner.objects.visible_to(request.user).select_related(
        "eve_alliance"
    )
    for alliance in alliances:
        owners.append(
            {
                "type": "alliance",
                "type_display": _("Alliance"),
                "id": alliance.eve_alliance.alliance_id,
                "name": alliance.eve_alliance.alliance_name,
                "portrait": lazy.get_alliance_logo_url(
                    alliance.eve_alliance.alliance_id,
                    size=64,
                    alliance_name=alliance.eve_alliance.alliance_name,
                    as_html=True,
                ),
                "active": alliance.active,
            }
        )

    # Sort by name
    owners.sort(key=lambda x: x["name"].lower())

    context = {
        "owners": owners,
        "title": _("Owner Overview"),
        "total_count": len(owners),
        "corporation_count": len([o for o in owners if o["type"] == "corporation"]),
        "alliance_count": len([o for o in owners if o["type"] == "alliance"]),
    }

    return render(
        request, "taxsystem/owner-overview.html", add_info_to_context(request, context)
    )


@login_required
@permission_required("taxsystem.basic_access")
def payments(request: WSGIRequest, owner_id: int = None):
    """Payments View"""
    if not owner_id:
        owner_id = request.user.profile.main_character.corporation_id

    owner, perms = get_owner(request, owner_id)

    if owner is None:
        messages.error(request, _("Owner not Found"))
        return redirect("taxsystem:index")

    if perms is False:
        messages.error(request, _("Permission Denied"))
        return redirect("taxsystem:index")

    # Determine owner type for context
    owner_type = get_owner_type_from_instance(owner)

    return generic_owner_payments(request, owner_id=owner_id, owner_type=owner_type)


@login_required
@permission_required("taxsystem.basic_access")
def own_payments(request: WSGIRequest, owner_id=None):
    """Own Payments View (Backwards-compatible wrapper)"""
    if not owner_id:
        owner_id = request.user.profile.main_character.corporation_id

    owner, perms = get_owner(request, owner_id)

    if owner is None:
        messages.error(request, _("Owner not Found"))
        return redirect("taxsystem:index")

    if perms is False:
        messages.error(request, _("Permission Denied"))
        return redirect("taxsystem:index")

    # Determine owner type for context
    owner_type = get_owner_type_from_instance(owner)
    return generic_owner_own_payments(request, owner_id=owner_id, owner_type=owner_type)


@login_required
@permission_required("taxsystem.basic_access")
def faq(request: WSGIRequest, owner_id=None):
    """FAQ View (Generic for Corporation and Alliance)"""
    if not owner_id:
        owner_id = request.user.profile.main_character.corporation_id

    # Get owner generically
    owner, perms = get_manage_owner(request, owner_id)
    perms = perms or get_character_permissions(
        request, request.user.profile.main_character.character_id
    )

    if owner is None:
        messages.error(request, _("Owner not Found"))
        return redirect("taxsystem:index")

    if perms is False:
        messages.error(request, _("Permission Denied"))
        return redirect("taxsystem:index")

    # Determine owner type for context
    owner_type = get_owner_type_from_instance(owner)
    context_key = get_owner_context_key(owner_type)

    corporations = CorporationOwner.objects.visible_to(request.user)

    context = {
        "owner_id": owner_id,
        context_key: owner_id,  # "corporation_id" or "alliance_id"
        "owner_type": owner_type,
        "title": _("FAQ"),
        "corporations": corporations,
    }
    context = add_info_to_context(request, context)

    return render(request, "taxsystem/faq.html", context=context)


@login_required
@permission_required("taxsystem.basic_access")
def account(request: WSGIRequest, owner_id=None, character_id=None):
    """Account View (Generic for Corporation and Alliance)"""
    if not owner_id:
        owner_id = request.user.profile.main_character.corporation_id

    if not character_id:
        character_id = request.user.profile.main_character.character_id

    user_profile = UserProfile.objects.filter(
        main_character__character_id=character_id
    ).first()

    if not user_profile:
        messages.error(request, _("No User found."))
        return redirect("taxsystem:index")

    try:
        owner, perms = get_manage_owner(request, owner_id)
        perms = perms or get_character_permissions(request, character_id)
    except AttributeError:
        messages.error(request, _("User has no main character set."))
        return redirect("taxsystem:index")

    if owner is None:
        messages.error(request, _("Owner not Found"))
        return redirect("taxsystem:index")

    if perms is False:
        messages.error(request, _("Permission Denied"))
        return redirect("taxsystem:index")

    payment_user = owner.payments_account_class.objects.filter(
        user__profile=user_profile,
        owner=owner,
    ).first()

    if not payment_user:
        messages.error(request, _("No Payment System User found."))
        return redirect("taxsystem:index")

    # Get member info
    try:
        member = Members.objects.get(character_id=character_id)
    except Members.DoesNotExist:
        member = None

    # Determine owner type for context
    owner_type = get_owner_type_from_instance(owner)
    context_key = get_owner_context_key(owner_type)

    context = {
        "title": _("Account"),
        "character_id": character_id,
        "owner_id": owner_id,
        context_key: owner_id,
        "owner_type": owner_type,
        "account": {
            "name": payment_user.name,
            "owner": owner,
            "corporation": owner,  # Backwards compatibility
            "status": payment_user.Status(payment_user.status).html(text=True),
            "deposit": (
                payment_user.deposit_html
                if payment_user.status != payment_user.Status.MISSING
                else "N/A"
            ),
            "has_paid": (
                payment_user.has_paid_icon(badge=True, text=True)
                if payment_user.status != payment_user.Status.MISSING
                else "N/A"
            ),
            "last_paid": (
                payment_user.last_paid
                if payment_user.status != payment_user.Status.MISSING
                else "N/A"
            ),
            "next_due": (
                payment_user.next_due
                if payment_user.status != payment_user.Status.MISSING
                else "N/A"
            ),
            "joined": member.joined if member else "N/A",
            "last_login": member.logon if member else "N/A",
        },
    }
    context = add_info_to_context(request, context)

    return render(request, "taxsystem/account.html", context=context)


@login_required
@permissions_required(
    [
        "taxsystem.manage_own_corp",
        "taxsystem.manage_corps",
        "taxsystem.manage_own_alliance",
        "taxsystem.manage_alliances",
    ]
)
def manage_owner(request: WSGIRequest, owner_id: int = None):
    """Manage View (Backwards-compatible wrapper)"""
    owner, perms = get_manage_owner(request, owner_id)

    if owner is None:
        messages.error(request, _("Owner not Found"))
        return redirect("taxsystem:index")

    if perms is False:
        messages.error(request, _("Permission Denied"))
        return redirect("taxsystem:index")

    # Determine owner type for context
    owner_type = get_owner_type_from_instance(owner)

    return generic_manage_owner(request, owner_id=owner_id, owner_type=owner_type)


@login_required
def manage_filter(request: WSGIRequest, owner_id: int):
    """Manage View"""
    owner, perms = get_manage_owner(request, owner_id)

    filter_sets = owner.filterset_class.objects.filter(owner=owner)
    context = {
        "corporation_id": owner_id if isinstance(owner, CorporationOwner) else None,
        "alliance_id": owner_id if isinstance(owner, AllianceOwner) else None,
        "owner_id": owner_id,
        "filter_sets": filter_sets,
        "title": _("Manage Filters"),
        "forms": {
            "filter": forms.AddJournalFilterForm(
                queryset=owner.filterset_class.objects.filter(owner=owner)
            ),
            "filter_set": forms.CreateFilterSetForm(),
            "delete_request": forms.MemberDeleteForm(),
        },
    }
    if perms is False:
        messages.error(request, _("You do not have permission to manage this owner."))
        return redirect("taxsystem:index")

    with transaction.atomic():
        form_add = forms.AddJournalFilterForm(
            data=request.POST,
            queryset=owner.filterset_class.objects.filter(owner=owner),
        )
        form_set = forms.CreateFilterSetForm(data=request.POST)

        if form_add.is_valid():
            queryset = form_add.cleaned_data["filter_set"]
            filter_type = form_add.cleaned_data["filter_type"]
            value = form_add.cleaned_data["value"]
            try:
                owner.filter_class.objects.create(
                    filter_set=queryset,
                    filter_type=filter_type,
                    value=value,
                )
            except IntegrityError:
                messages.error(request, _("A filter with this name already exists."))
                return redirect("taxsystem:manage_filter", owner_id=owner_id)
            except Exception as e:  # pylint: disable=broad-except
                messages.error(
                    request, _("Something went wrong, please try again later.")
                )
                logger.error("Error creating journal filter: %s", e)
                return redirect("taxsystem:manage_filter", owner_id=owner_id)

        if form_set.is_valid():
            name = form_set.cleaned_data["name"]
            description = form_set.cleaned_data["description"]
            try:
                owner.filterset_class.objects.create(
                    owner=owner,
                    name=name,
                    description=description,
                )
            except IntegrityError:
                messages.error(
                    request, _("A filter set with this name already exists.")
                )
                return redirect("taxsystem:manage_filter", owner_id=owner_id)
            except Exception as e:  # pylint: disable=broad-except
                messages.error(
                    request, _("Something went wrong, please try again later.")
                )
                logger.error("Error creating journal filter set: %s", e)
                return render(request, "taxsystem/manage-filter.html", context=context)

    return render(request, "taxsystem/manage-filter.html", context=context)


@login_required
def switch_filterset(request: WSGIRequest, owner_id: int, filter_set_id: int):
    """Deactivate Filter Set View"""
    owner, perms = get_manage_owner(request, owner_id)

    if perms is False:
        messages.error(request, _("You do not have permission to manage this owner."))
        return redirect("taxsystem:index")

    filter_set = get_object_or_404(
        owner.filterset_class.objects.filter(owner=owner), id=filter_set_id
    )
    filter_sets = owner.filterset_class.objects.filter(owner=owner)

    filter_set.enabled = not filter_set.enabled
    filter_set.save()

    context = {
        "owner_id": owner_id,
        "filter_sets": filter_sets,
        "forms": {
            "filter": forms.AddJournalFilterForm(
                queryset=owner.filterset_class.objects.filter(owner=owner)
            ),
            "filter_set": forms.CreateFilterSetForm(),
        },
        "title": _("Deactivate Filter Set"),
    }
    context = add_info_to_context(request, context)

    messages.success(
        request, _(f"Filter set switched to {filter_set.enabled} successfully.")
    )
    return redirect("taxsystem:manage_filter", owner_id=owner_id)


@login_required
def delete_filterset(request: WSGIRequest, owner_id: int, filter_set_id: int):
    """Delete Filter Set View"""
    owner, perms = get_manage_owner(request, owner_id)

    if perms is False:
        messages.error(request, _("You do not have permission to manage this owner."))
        return redirect("taxsystem:index")

    filter_set = get_object_or_404(
        owner.filterset_class.objects.filter(owner=owner), id=filter_set_id
    )
    filter_sets = owner.filterset_class.objects.filter(owner=owner)

    filter_set.delete()
    msg = _(f"{filter_set.name} from {owner.name} deleted")
    owner.admin_history_class(
        user=request.user,
        owner=owner,
        action=owner.admin_history_class.Actions.DELETE,
        comment=msg,
    ).save()
    messages.success(request, _("Filter set deleted successfully."))

    context = {
        "owner_id": owner_id,
        "filter_sets": filter_sets,
        "forms": {
            "filter": forms.AddJournalFilterForm(
                queryset=owner.filterset_class.objects.filter(owner=owner)
            ),
            "filter_set": forms.CreateFilterSetForm(),
        },
        "title": _("Delete Filter Set"),
    }
    context = add_info_to_context(request, context)

    return redirect("taxsystem:manage_filter", owner_id=owner_id)


@login_required
@require_POST
def delete_filter(request: WSGIRequest, owner_id: int, filter_pk: int):
    msg = _("Invalid Method")
    owner, perms = get_manage_owner(request, owner_id)

    if not perms:
        msg = _("Permission Denied")
        return JsonResponse(
            data={"success": False, "message": msg}, status=403, safe=False
        )

    form = forms.FilterDeleteForm(data=request.POST)
    if form.is_valid():
        filter_obj = owner.filter_class.objects.get(
            filter_set__owner=owner, pk=filter_pk
        )
        if filter_obj:
            msg = _(
                f"{filter_obj.filter_type}({filter_obj.value}) from {filter_obj.filter_set} deleted - {form.cleaned_data['delete_reason']}"
            )
            filter_obj.delete()
            owner.admin_history_class(
                user=request.user,
                owner=owner,
                action=owner.admin_history_class.Actions.DELETE,
                comment=msg,
            ).save()
            return JsonResponse(
                data={"success": True, "message": msg}, status=200, safe=False
            )
    return JsonResponse(data={"success": False, "message": msg}, status=400, safe=False)


@login_required
def edit_filterset(request: WSGIRequest, owner_id: int, filter_set_id: int):
    """Edit Filter Set View"""
    owner, perms = get_manage_owner(request, owner_id)

    if perms is False:
        messages.error(request, _("You do not have permission to manage this owner."))
        return redirect("taxsystem:index")

    edit_set = get_object_or_404(
        owner.filterset_class.filter(owner=owner), id=filter_set_id
    )
    filter_sets = owner.filterset_class.filter(owner=owner)

    if request.method == "POST":
        form = forms.EditFilterSetForm(request.POST, instance=edit_set)
        if form.is_valid():
            form.save()
            messages.success(request, _("Filter set updated successfully."))
            return redirect("taxsystem:manage_filter", owner_id=owner_id)
    else:
        form = forms.EditFilterSetForm(instance=edit_set)

    context = {
        "owner_id": owner_id,
        "filter_sets": filter_sets,
        "forms": {
            "edit_filter_set": form,
        },
        "title": _("Edit Filter Set"),
    }
    context = add_info_to_context(request, context)

    return render(request, "taxsystem/manage-filter.html", context=context)


@login_required
@permission_required("taxsystem.create_access")
@token_required(scopes=CorporationOwner.get_esi_scopes())
def add_corp(request, token):
    char = get_object_or_404(EveCharacter, character_id=token.character_id)
    corp, __ = EveCorporationInfo.objects.get_or_create(
        corporation_id=char.corporation_id,
        defaults={
            "member_count": 0,
            "corporation_ticker": char.corporation_ticker,
            "corporation_name": char.corporation_name,
        },
    )

    owner, created = CorporationOwner.objects.update_or_create(
        eve_corporation=corp,
        defaults={
            "name": char.corporation_name,
            "active": True,
        },
    )

    if created:
        CorporationAdminHistory(
            user=request.user,
            owner=owner,
            action=CorporationAdminHistory.Actions.ADD,
            comment=_("Added to Tax System"),
        ).save()

    tasks.update_corporation.apply_async(
        args=[owner.pk], kwargs={"force_refresh": True}, priority=6
    )
    msg = _("{corporation_name} successfully added/updated to Tax System").format(
        corporation_name=char.corporation_name,
    )
    messages.info(request, msg)
    return redirect("taxsystem:index")


@login_required
@permission_required("taxsystem.create_access")
@token_required(scopes=CorporationOwner.get_esi_scopes())
def add_alliance(request, token):
    char = get_object_or_404(EveCharacter, character_id=token.character_id)
    tax_corp = get_object_or_404(
        CorporationOwner, eve_corporation__corporation_id=char.corporation_id
    )

    ally, __ = EveAllianceInfo.objects.get_or_create(
        alliance_id=char.alliance_id,
        defaults={
            "member_count": 0,
            "alliance_ticker": char.alliance_ticker,
            "alliance_name": char.alliance_name,
        },
    )

    owner_alliance, created = AllianceOwner.objects.update_or_create(
        eve_alliance=ally,
        defaults={
            "corporation": tax_corp,
            "name": char.alliance_name,
            "active": True,
        },
    )

    if created:
        AllianceAdminHistory(
            user=request.user,
            owner=owner_alliance,
            action=AllianceAdminHistory.Actions.ADD,
            comment=_("Added Alliance to Tax System with Corporation {corp}").format(
                corp=tax_corp.name
            ),
        ).save()

    tasks.update_alliance.apply_async(
        args=[owner_alliance.pk], kwargs={"force_refresh": True}, priority=6
    )
    msg = _("{alliance_name} successfully added/updated to Tax System").format(
        alliance_name=char.alliance_name,
    )
    messages.info(request, msg)
    return redirect("taxsystem:index")


@login_required
@require_POST
def approve_payment(request: WSGIRequest, owner_id: int, payment_pk: int):
    msg = _("Invalid Method")
    owner, perms = get_manage_owner(request, owner_id)

    if not perms:
        msg = _("Permission Denied")
        return JsonResponse(
            data={"success": False, "message": msg}, status=403, safe=False
        )

    try:
        with transaction.atomic():
            form = forms.PaymentAcceptForm(data=request.POST)
            if form.is_valid():
                reason = form.cleaned_data["accept_info"]
                payment = owner.payments_class.objects.get(
                    account__owner=owner, pk=payment_pk
                )
                if payment.is_pending or payment.is_needs_approval:
                    msg = _(
                        "Payment ID: {pid} - Amount: {amount} - Name: {name} approved"
                    ).format(
                        pid=payment.pk,
                        amount=intcomma(payment.amount),
                        name=payment.name,
                    )
                    payment.request_status = owner.payments_class.RequestStatus.APPROVED
                    payment.reviser = request.user.profile.main_character.character_name
                    payment.save()

                    payment_account = owner.payments_account_class.objects.get(
                        owner=owner, user=payment.account.user
                    )
                    payment_account.deposit += payment.amount
                    payment_account.save()
                    owner.payments_history_class(
                        user=request.user,
                        payment=payment,
                        action=owner.payments_history_class.Actions.STATUS_CHANGE,
                        comment=reason,
                        new_status=owner.payments_class.RequestStatus.APPROVED,
                    ).save()
                    return JsonResponse(
                        data={"success": True, "message": msg}, status=200, safe=False
                    )
    except IntegrityError:
        msg = _("Transaction failed. Please try again.")
    return JsonResponse(data={"success": False, "message": msg}, status=400, safe=False)


@login_required
@require_POST
def undo_payment(request: WSGIRequest, owner_id: int, payment_pk: int):
    msg = _("Invalid Method")
    owner, perms = get_manage_owner(request, owner_id)

    if not perms:
        msg = _("Permission Denied")
        return JsonResponse(
            data={"success": False, "message": msg}, status=403, safe=False
        )

    try:
        with transaction.atomic():
            form = forms.PaymentUndoForm(data=request.POST)
            if form.is_valid():
                reason = form.cleaned_data["undo_reason"]
                payment = owner.payments_class.objects.get(
                    account__owner=owner, pk=payment_pk
                )
                if payment.is_approved or payment.is_rejected:
                    msg = _(
                        "Payment ID: {pid} - Amount: {amount} - Name: {name} undone"
                    ).format(
                        pid=payment.pk,
                        amount=intcomma(payment.amount),
                        name=payment.name,
                    )
                    # Ensure that the payment is not rejected
                    if not payment.is_rejected:
                        payment_account = owner.payments_account_class.objects.get(
                            owner=owner, user=payment.account.user
                        )
                        payment_account.deposit -= payment.amount
                        payment_account.save()
                    payment.request_status = owner.payments_class.RequestStatus.PENDING
                    payment.reviser = ""
                    payment.save()
                    owner.payments_history_class(
                        user=request.user,
                        payment=payment,
                        action=owner.payments_history_class.Actions.STATUS_CHANGE,
                        comment=reason,
                        new_status=owner.payments_class.RequestStatus.PENDING,
                    ).save()
                    return JsonResponse(
                        data={"success": True, "message": msg}, status=200, safe=False
                    )
    except IntegrityError:
        msg = _("Transaction failed. Please try again.")
    return JsonResponse(data={"success": False, "message": msg}, status=400, safe=False)


@login_required
@require_POST
def reject_payment(request: WSGIRequest, owner_id: int, payment_pk: int):
    msg = _("Invalid Method")
    owner, perms = get_manage_owner(request, owner_id)

    if not perms:
        msg = _("Permission Denied")
        return JsonResponse(
            data={"success": False, "message": msg}, status=403, safe=False
        )

    try:
        with transaction.atomic():
            form = forms.PaymentRejectForm(data=request.POST)
            if form.is_valid():
                reason = form.cleaned_data["reject_reason"]
                payment = owner.payments_class.objects.get(
                    account__owner=owner, pk=payment_pk
                )
                if payment.is_pending or payment.is_needs_approval:
                    payment.request_status = owner.payments_class.RequestStatus.REJECTED
                    payment.reviser = request.user.profile.main_character.character_name
                    payment.save()

                    payment_account = owner.payments_account_class.objects.get(
                        owner=owner, user=payment.account.user
                    )
                    payment_account.save()
                    msg = _(
                        "Payment ID: {pid} - Amount: {amount} - Name: {name} rejected"
                    ).format(
                        pid=payment.pk,
                        amount=intcomma(payment.amount),
                        name=payment.name,
                    )

                    owner.payments_history_class(
                        user=request.user,
                        payment=payment,
                        action=owner.payments_history_class.Actions.STATUS_CHANGE,
                        comment=reason,
                        new_status=owner.payments_class.RequestStatus.REJECTED,
                    ).save()
                    return JsonResponse(
                        data={"success": True, "message": msg}, status=200, safe=False
                    )
    except IntegrityError:
        msg = _("Transaction failed. Please try again.")
    return JsonResponse(data={"success": False, "message": msg}, status=400, safe=False)


@login_required
@require_POST
def delete_payment(request: WSGIRequest, owner_id: int, payment_pk: int):
    msg = _("Invalid Method")
    owner, perms = get_manage_owner(request, owner_id)

    if not perms:
        msg = _("Permission Denied")
        return JsonResponse(
            data={"success": False, "message": msg}, status=403, safe=False
        )

    try:
        with transaction.atomic():
            form = forms.PaymentDeleteForm(data=request.POST)
            if form.is_valid():
                reason = form.cleaned_data["delete_reason"]
                payment = owner.payments_class.objects.get(
                    account__owner=owner, pk=payment_pk
                )

                if payment.entry_id != 0:  # Prevent deletion of ESI imported payments
                    msg = _(
                        "Payment ID: {pid} - Amount: {amount} - Name: {name} deletion failed - ESI imported payments cannot be deleted"
                    ).format(
                        pid=payment.pk,
                        amount=intcomma(payment.amount),
                        name=payment.name,
                    )
                    return JsonResponse(
                        data={"success": False, "message": msg}, status=400, safe=False
                    )

                msg = _(
                    "Payment ID: {pid} - Amount: {amount} - Name: {name} deleted - {reason}"
                ).format(
                    pid=payment.pk,
                    amount=intcomma(payment.amount),
                    name=payment.name,
                    reason=reason,
                )

                # Refund if approved
                if payment.is_approved:
                    payment_user = owner.payments_account_class.objects.get(
                        owner=owner, user=payment.account.user
                    )
                    payment_user.deposit -= payment.amount
                    payment_user.save()

                # Delete Payment
                payment.delete()

                # Log Admin Action
                owner.admin_history_class(
                    user=request.user,
                    owner=owner,
                    action=owner.admin_history_class.Actions.DELETE,
                    comment=msg,
                ).save()

                return JsonResponse(
                    data={"success": True, "message": msg}, status=200, safe=False
                )
    except IntegrityError:
        msg = _("Transaction failed. Please try again.")
    return JsonResponse(data={"success": False, "message": msg}, status=400, safe=False)


@login_required
@require_POST
def add_payment(request: WSGIRequest, owner_id: int, payment_system_pk: int):
    msg = _("Invalid Method")
    owner, perms = get_manage_owner(request, owner_id)

    if not perms:
        msg = _("Permission Denied")
        return JsonResponse(
            data={"success": False, "message": msg}, status=403, safe=False
        )

    try:
        with transaction.atomic():
            form = forms.PaymentAddForm(data=request.POST)
            if form.is_valid():
                amount = form.cleaned_data["amount"]
                reason = form.cleaned_data["add_reason"]
                payment_account = owner.payments_account_class.objects.get(
                    owner=owner, pk=payment_system_pk
                )

                payment = owner.payments_class(
                    name=payment_account.user.username,
                    entry_id=0,  # Manual Entry
                    amount=amount,
                    account=payment_account,
                    date=timezone.now(),
                    reason=reason,
                    request_status=owner.payments_class.RequestStatus.APPROVED,
                    reviser=request.user.username,
                    owner_id=owner_id,
                )
                payment.save()
                payment_account.deposit += amount
                payment_account.save()

                msg = _(
                    "Payment ID: {pid} - Amount: {amount} - Name: {name} added"
                ).format(
                    pid=payment.pk,
                    amount=intcomma(payment.amount),
                    name=payment.name,
                )

                # Log Payment Action
                owner.payments_history_class(
                    user=request.user,
                    payment=payment,
                    action=owner.payments_history_class.Actions.PAYMENT_ADDED,
                    comment=reason,
                    new_status=owner.payments_class.RequestStatus.APPROVED,
                ).save()

                # Log Admin Action
                owner.admin_history_class(
                    user=request.user,
                    owner=owner,
                    action=owner.admin_history_class.Actions.ADD,
                    comment=msg,
                ).save()
                return JsonResponse(
                    data={"success": True, "message": msg}, status=200, safe=False
                )
    except IntegrityError:
        msg = _("Transaction failed. Please try again.")
    return JsonResponse(data={"success": False, "message": msg}, status=400, safe=False)


@login_required
@require_POST
def switch_user(request: WSGIRequest, owner_id: int, payment_system_pk: int):
    msg = _("Invalid Method")
    owner, perms = get_manage_owner(request, owner_id)

    if not perms:
        msg = _("Permission Denied")
        return JsonResponse(
            data={"success": False, "message": msg}, status=403, safe=False
        )

    try:
        with transaction.atomic():
            form = forms.TaxSwitchUserForm(data=request.POST)
            if form.is_valid():
                # Get Payment System User
                payment_system = owner.payments_account_class.objects.get(
                    owner=owner, pk=payment_system_pk
                )
                # Toggle Active Status
                if payment_system.is_active:
                    payment_system.status = payment_system.Status.DEACTIVATED
                    msg = _("Payment System User: %s deactivated") % payment_system.name
                else:
                    payment_system.status = payment_system.Status.ACTIVE
                    msg = _("Payment System User: %s activated") % payment_system.name

                # Log Admin Action
                owner.admin_history_class(
                    user=request.user,
                    owner=owner,
                    action=owner.admin_history_class.Actions.CHANGE,
                    comment=msg,
                ).save()
                payment_system.save()
            return JsonResponse(
                data={"success": True, "message": msg}, status=200, safe=False
            )
    except IntegrityError:
        msg = _("Transaction failed. Please try again.")
    return JsonResponse(data={"success": False, "message": msg}, status=400, safe=False)


@login_required
@csrf_exempt
def update_tax_amount(request: WSGIRequest, owner_id: int):
    if request.method == "POST":
        value = float(request.POST.get("value"))
        msg = _("Please enter a valid number")
        try:
            if value < 0:
                return JsonResponse({"message": msg}, status=400)
        except ValueError:
            return JsonResponse({"message": msg}, status=400)

        owner, perms = get_manage_owner(request, owner_id)

        logger.debug(
            f"Updating tax amount for owner ID {owner_id} to {value}. Permissions: {perms}"
        )

        if not perms:
            return JsonResponse({"message": _("Permission Denied")}, status=403)

        try:
            owner.tax_amount = value
            owner.save()
            msg = _(f"Tax Amount from {owner.name} updated to {value}")
            owner.admin_history_class(
                user=request.user,
                owner=owner,
                action=owner.admin_history_class.Actions.CHANGE,
                comment=msg,
            ).save()
            logger.debug(f"Tax amount updated successfully for owner ID {owner_id}")
        except ValidationError:
            return JsonResponse({"message": msg}, status=400)
        return JsonResponse({"message": msg}, status=200)
    return JsonResponse({"message": _("Invalid request method")}, status=405)


@login_required
@csrf_exempt
def update_tax_period(request: WSGIRequest, owner_id: int):
    if request.method == "POST":
        value = int(request.POST.get("value"))
        msg = _("Please enter a valid number")
        try:
            if value < 0:
                return JsonResponse({"message": msg}, status=400)
        except ValueError:
            return JsonResponse({"message": msg}, status=400)

        owner, perms = get_manage_owner(request, owner_id)

        if not perms:
            return JsonResponse({"message": _("Permission Denied")}, status=403)

        try:
            owner.tax_period = value
            owner.save()
            msg = _(f"Tax Period from {owner.name} updated to {value}")
            owner.admin_history_class(
                user=request.user,
                owner=owner,
                action=owner.admin_history_class.Actions.CHANGE,
                comment=msg,
            ).save()
        except ValidationError:
            return JsonResponse({"message": msg}, status=400)
        return JsonResponse({"message": msg}, status=200)
    return JsonResponse({"message": _("Invalid request method")}, status=405)


@login_required
@require_POST
def delete_member(request: WSGIRequest, corporation_id: int, member_pk: int):
    msg = _("Invalid Method")
    corp, perms = get_manage_corporation(request, corporation_id)
    if not perms:
        msg = _("Permission Denied")
        return JsonResponse(
            data={"success": False, "message": msg}, status=403, safe=False
        )

    form = forms.MemberDeleteForm(data=request.POST)
    if form.is_valid():
        reason = form.cleaned_data["delete_reason"]
        member = Members.objects.get(owner=corp, pk=member_pk)
        if member.is_missing:
            msg = _(f"Member {member.character_name} deleted - {reason}")
            member.delete()
            CorporationAdminHistory(
                user=request.user,
                owner=corp,
                action=CorporationAdminHistory.Actions.DELETE,
                comment=msg,
            ).save()
            return JsonResponse(
                data={"success": True, "message": msg}, status=200, safe=False
            )
    return JsonResponse(data={"success": False, "message": msg}, status=400, safe=False)


# =============================================================================
# Generic Owner Views (Corporation and Alliance Unified)
# =============================================================================


@login_required
def generic_manage_owner(
    request: WSGIRequest, owner_id: int = None, owner_type: str = "corporation"
):
    """
    Unified Owner Management View for Corporation and Alliance.

    Args:
        request: Django request object
        owner_id: Owner ID (corporation_id or alliance_id)
        owner_type: "corporation" or "alliance"

    Returns:
        Rendered manage template
    """
    # 1. Default ID resolution
    if owner_id is None:
        owner_id = get_default_owner_id(request, owner_type)
        if owner_id is None:
            messages.error(
                request,
                _("No default {owner_type} found.").format(
                    owner_type=get_owner_display_name(owner_type)
                ),
            )
            return redirect("taxsystem:index")

    # 2. Get owner and permissions
    owner, perms = get_manage_owner(request, owner_id)

    # 3. Permission checks
    if perms is False:
        messages.error(
            request,
            _("You do not have permission to manage this {owner_type}.").format(
                owner_type=get_owner_display_name(owner_type).lower()
            ),
        )
        return redirect("taxsystem:index")

    if owner is None:
        messages.error(
            request,
            _("{owner_type} not Found.").format(
                owner_type=get_owner_display_name(owner_type)
            ),
        )
        return redirect("taxsystem:index")

    # 4. Build context
    context_key = get_owner_context_key(owner_type)
    template = get_owner_template("manage", owner_type)

    context = {
        context_key: owner_id,  # "corporation_id" or "alliance_id"
        "owner_id": owner_id,  # Generic key for templates
        "owner_type": owner_type,  # For conditional template logic
        "corporations": CorporationOwner.objects.visible_to(request.user),
        "title": (
            _("Corporation Tax System")
            if owner_type == "corporation"
            else _("Alliance Tax System")
        ),
        "manage_filter_url": reverse(
            "taxsystem:manage_filter", kwargs={"owner_id": owner_id}
        ),
        "forms": {
            "accept_request": forms.PaymentAcceptForm(),
            "reject_request": forms.PaymentRejectForm(),
            "add_request": forms.PaymentAddForm(),
            "payment_delete_request": forms.PaymentDeleteForm(),
            "undo_request": forms.PaymentUndoForm(),
            "switchuser_request": forms.TaxSwitchUserForm(),
            "delete_request": forms.MemberDeleteForm(),
        },
    }
    context = add_info_to_context(request, context)

    return render(request, template, context=context)


@login_required
@permission_required("taxsystem.basic_access")
def generic_owner_payments(
    request: WSGIRequest, owner_id: int = None, owner_type: str = "corporation"
):
    """
    Unified Payments View for Corporation and Alliance.

    Args:
        request: Django request object
        owner_id: Owner ID (corporation_id or alliance_id)
        owner_type: "corporation" or "alliance"

    Returns:
        Rendered payments template
    """
    # 1. Default ID resolution
    if owner_id is None:
        owner_id = get_default_owner_id(request, owner_type)
        if owner_id is None:
            messages.error(
                request,
                _("No default {owner_type} found.").format(
                    owner_type=get_owner_display_name(owner_type)
                ),
            )
            return redirect("taxsystem:index")

    # 2. Get owner
    if owner_type == "corporation":
        owner = get_corporation(request, owner_id)
    else:
        owner = get_alliance(request, owner_id)

    if owner is None:
        messages.error(
            request,
            _("No {owner_type} found.").format(
                owner_type=get_owner_display_name(owner_type)
            ),
        )

    # 3. Build context
    context_key = get_owner_context_key(owner_type)
    template = get_owner_template("payments", owner_type)

    corporations = CorporationOwner.objects.visible_to(request.user)

    context = {
        context_key: owner_id,  # "corporation_id" or "alliance_id"
        "owner_id": owner_id,  # Generic key
        "owner_type": owner_type,
        "title": _("Payments"),
        "forms": {
            "add_request": forms.PaymentAddForm(),
            "payment_delete_request": forms.PaymentDeleteForm(),
            "accept_request": forms.PaymentAcceptForm(),
            "reject_request": forms.PaymentRejectForm(),
            "undo_request": forms.PaymentUndoForm(),
        },
        "corporations": corporations,
    }
    context = add_info_to_context(request, context)

    return render(request, template, context=context)


@login_required
@permission_required("taxsystem.basic_access")
def generic_owner_own_payments(
    request: WSGIRequest, owner_id: int = None, owner_type: str = "corporation"
):
    """
    Unified Own Payments View for Corporation and Alliance.

    Args:
        request: Django request object
        owner_id: Owner ID (corporation_id or alliance_id)
        owner_type: "corporation" or "alliance"

    Returns:
        Rendered own payments template
    """
    # 1. Default ID resolution
    if owner_id is None:
        owner_id = get_default_owner_id(request, owner_type)
        if owner_id is None:
            messages.error(
                request,
                _("No default {owner_type} found.").format(
                    owner_type=get_owner_display_name(owner_type)
                ),
            )
            return redirect("taxsystem:index")

    # 2. Get owner and permissions
    if owner_type == "corporation":
        owner = get_corporation(request, owner_id)
    else:
        owner = get_alliance(request, owner_id)

    if owner is None:
        messages.error(
            request,
            _("No {owner_type} found.").format(
                owner_type=get_owner_display_name(owner_type)
            ),
        )
        return redirect("taxsystem:index")

    # 3. Build context
    context_key = get_owner_context_key(owner_type)
    template = get_owner_template("own_payments", owner_type)

    corporations = CorporationOwner.objects.visible_to(request.user)

    context = {
        context_key: owner_id,  # "corporation_id" or "alliance_id"
        "owner_id": owner_id,  # Generic key
        "owner_type": owner_type,
        "title": _("Own Payments") if owner_type == "alliance" else _("My Payments"),
        "corporations": corporations,
    }
    context = add_info_to_context(request, context)

    return render(request, template, context=context)



# Third Party
from ninja import NinjaAPI, Schema

# Django
from django.shortcuts import get_object_or_404, render
from django.utils.translation import gettext_lazy as _

# Alliance Auth
from allianceauth.services.hooks import get_extension_logger

# Alliance Auth (External Libs)
from app_utils.logging import LoggerAddTag

# AA TaxSystem
from taxsystem import __title__
from taxsystem.api.helpers import core
from taxsystem.api.helpers.manage import manage_payments
from taxsystem.api.schema import (
    CharacterSchema,
    LogHistorySchema,
    PaymentSchema,
    RequestStatusSchema,
)
from taxsystem.helpers.lazy import get_character_portrait_url
from taxsystem.models.corporation import (
    CorporationPaymentAccount,
)

logger = LoggerAddTag(get_extension_logger(__name__), __title__)


class PaymentAccountSchema(Schema):
    account_id: int
    account_name: str
    character: CharacterSchema
    payment_pool: int
    payment_status: str


class PaymentDetailsSchema(Schema):
    account: PaymentAccountSchema | None
    payment: PaymentSchema | None
    payment_histories: list[LogHistorySchema] | None


class PaymentsDetailsResponse(Schema):
    title: str | None = None
    entity_pk: int
    entity_type: str
    payment_details: PaymentDetailsSchema | list[PaymentDetailsSchema]


class PaymentsCharacterResponse(PaymentsDetailsResponse):
    payment_details: list[PaymentSchema]
    character: CharacterSchema


class CharacterApiEndpoints:
    tags = ["Character Tax System"]

    # pylint: disable=too-many-statements
    def __init__(self, api: NinjaAPI):
        @api.get(
            "owner/{owner_id}/character/{character_id}/payment/{pk}/view/details/",
            response={200: PaymentsDetailsResponse, 403: dict, 404: dict},
            tags=self.tags,
        )
        # pylint: disable=too-many-locals
        def get_payment_details(request, owner_id: int, character_id: int, pk: int):
            owner, perms = core.get_manage_owner(request, owner_id)
            perms = perms or core.get_character_permissions(request, character_id)

            # pylint: disable=duplicate-code
            if owner is None:
                return 404, {"error": _("Owner Not Found")}

            # pylint: disable=duplicate-code
            if perms is False:
                return 403, {"error": _("Permission Denied")}

            payment = get_object_or_404(owner.payments_class, pk=pk)
            account = get_object_or_404(
                owner.payments_account_class,
                user=payment.account.user,
                owner=owner,
            )

            response_payment_histories: list[LogHistorySchema] = []
            payments_history = owner.payments_history_class.objects.filter(
                payment=payment,
            ).order_by("-date")

            # Create the payment account
            response_account = PaymentAccountSchema(
                account_id=account.pk,
                account_name=account.name,
                character=CharacterSchema(
                    character_id=payment.character_id,
                    character_name=payment.account.name,
                    character_portrait=get_character_portrait_url(
                        payment.character_id, size=32, as_html=True
                    ),
                    corporation_id=account.owner.pk,
                    corporation_name=account.owner.name,
                ),
                payment_pool=account.deposit,
                payment_status=CorporationPaymentAccount.Status(account.status).html(),
            )

            # Create a list for the payment histories
            for log in payments_history:
                response_log: LogHistorySchema = LogHistorySchema(
                    log_id=log.pk,
                    reviser=log.user.username if log.user else _("System"),
                    date=log.date.strftime("%Y-%m-%d %H:%M:%S"),
                    action=log.get_action_display(),
                    comment=log.get_comment_display(),
                    status=log.get_new_status_display(),
                )
                response_payment_histories.append(response_log)

            response_request_status = RequestStatusSchema(
                status=payment.get_request_status_display(),
                color=payment.RequestStatus(payment.request_status).color(),
            )

            # Create the payment
            response_payment = PaymentSchema(
                payment_id=payment.pk,
                amount=payment.amount,
                date=payment.formatted_payment_date,
                request_status=response_request_status,
                division_name=payment.division_name,
                reason=payment.reason,
                reviser=payment.reviser,
            )

            # Create the payment details
            payment_details: PaymentDetailsSchema = PaymentDetailsSchema(
                account=response_account,
                payment=response_payment,
                payment_histories=response_payment_histories,
            )

            # Create the response
            paymentdetails_response = PaymentsDetailsResponse(
                entity_pk=owner_id,
                entity_type="character",
                payment_details=payment_details,
            )

            return render(
                request=request,
                template_name="taxsystem/modals/view_payment_details.html",
                context=paymentdetails_response.dict(),
            )

        @api.get(
            "owner/{owner_id}/character/{character_id}/view/payments/",
            response={200: list, 403: dict, 404: dict},
            tags=self.tags,
        )
        def get_member_payments(request, owner_id: int, character_id: int):
            owner, perms = core.get_manage_owner(request, owner_id)

            # pylint: disable=duplicate-code
            if owner is None:
                return 404, {"error": _("Corporation Not Found")}

            # pylint: disable=duplicate-code
            if perms is False:
                return 403, {"error": _("Permission Denied")}

            # Filter the last 10000 payments by character
            payments = owner.payments_class.objects.filter(
                account__owner=owner,
                account__user__profile__main_character__character_id=character_id,
                owner_id=owner.eve_id,
            ).order_by("-date")[:10000]

            if not payments:
                return 404, {"error": _("No Payments Found")}

            response_payments_list: list[PaymentSchema] = []
            for payment in payments:
                try:
                    character_id = (
                        payment.account.user.profile.main_character.character_id
                    )
                    portrait = get_character_portrait_url(
                        character_id, size=32, as_html=True
                    )
                except AttributeError:
                    portrait = ""

                # Create the actions
                actions_html = manage_payments(
                    request=request, perms=perms, payment=payment
                )

                # pylint: disable=duplicate-code
                # Create the request status
                response_request_status = RequestStatusSchema(
                    status=payment.get_request_status_display(),
                    color=payment.RequestStatus(payment.request_status).color(),
                )

                response_payment = PaymentSchema(
                    payment_id=payment.pk,
                    amount=payment.amount,
                    date=payment.formatted_payment_date,
                    request_status=response_request_status,
                    division_name=payment.division_name,
                    reason=payment.reason,
                    actions=actions_html,
                    reviser=payment.reviser,
                )
                response_payments_list.append(response_payment)

            character_payments_response = PaymentsCharacterResponse(
                entity_pk=character_id,
                entity_type="character",
                payment_details=response_payments_list,
                character=CharacterSchema(
                    character_id=character_id,
                    character_name=payments[0].account.name,
                    character_portrait=portrait,
                ),
            )

            return render(
                request,
                "taxsystem/modals/view_character_payments.html",
                context=character_payments_response.dict(),
            )



# Third Party
from ninja import NinjaAPI, Schema

# Django
from django.shortcuts import render
from django.utils.translation import gettext as _

# Alliance Auth
from allianceauth.services.hooks import get_extension_logger

# Alliance Auth (External Libs)
from app_utils.logging import LoggerAddTag

# AA TaxSystem
from taxsystem import __title__
from taxsystem.api.helpers import core
from taxsystem.api.helpers.common import (
    build_admin_logs_response_list,
    build_filters_response_list,
    build_payment_accounts_response_list,
    calculate_activity,
    create_dashboard_common_data,
    create_member_response_data,
)
from taxsystem.api.helpers.manage import (
    generate_member_delete_button,
)
from taxsystem.api.schema import (
    AdminHistorySchema,
    BaseDashboardResponse,
    CorporationSchema,
    FilterModelSchema,
    MembersSchema,
    PaymentSystemSchema,
)
from taxsystem.helpers import lazy
from taxsystem.models.corporation import (
    CorporationAdminHistory,
    Members,
)
from taxsystem.models.wallet import CorporationWalletDivision

logger = LoggerAddTag(get_extension_logger(__name__), __title__)


class MembersResponse(Schema):
    corporation: list[MembersSchema]


class DashboardResponse(BaseDashboardResponse):
    owner: CorporationSchema


class PaymentSystemResponse(Schema):
    owner: list[PaymentSystemSchema]


class AdminLogResponse(Schema):
    corporation: list[AdminHistorySchema]


class AdminApiEndpoints:
    tags = ["Admin"]

    # pylint: disable=too-many-statements
    def __init__(self, api: NinjaAPI):
        @api.get(
            "corporation/{corporation_id}/view/dashboard/",
            response={200: DashboardResponse, 403: dict, 404: dict},
            tags=self.tags,
        )
        # pylint: disable=too-many-locals
        def get_dashboard(request, corporation_id: int):
            owner, perms = core.get_manage_corporation(request, corporation_id)

            if owner is None:
                return 404, {"error": _("Corporation Not Found")}

            if perms is False:
                return 403, {"error": _("Permission Denied")}

            divisions = CorporationWalletDivision.objects.filter(corporation=owner)

            corporation_logo = lazy.get_corporation_logo_url(
                corporation_id, size=64, as_html=True
            )

            # Create common dashboard data
            common_data = create_dashboard_common_data(owner, divisions)

            # Calculate activity
            activity = calculate_activity(owner, corporation_id)

            dashboard_response = DashboardResponse(
                owner=CorporationSchema(
                    owner_id=owner.eve_corporation.corporation_id,
                    owner_name=owner.eve_corporation.corporation_name,
                    owner_type="corporation",
                    corporation_id=owner.eve_corporation.corporation_id,
                    corporation_name=owner.eve_corporation.corporation_name,
                    corporation_portrait=corporation_logo,
                    corporation_ticker=owner.eve_corporation.corporation_ticker,
                ),
                activity=activity,
                **common_data,
            )
            return dashboard_response

        @api.get(
            "corporation/{corporation_id}/view/members/",
            response={200: MembersResponse, 403: dict, 404: dict},
            tags=self.tags,
        )
        def get_members(request, corporation_id: int):
            owner, perms = core.get_manage_corporation(request, corporation_id)

            if owner is None:
                return 404, {"error": _("Corporation Not Found")}

            if perms is False:
                return 403, {"error": _("Permission Denied")}

            # Get Members
            members = (
                Members.objects.filter(owner=owner)
                .select_related("owner")
                .order_by("character_name")
            )

            response_members_list: list[MembersSchema] = []
            for member in members:
                actions = ""
                # Create the delete button if member is missing
                if perms and member.is_missing:
                    actions = generate_member_delete_button(member=member)

                member_data = create_member_response_data(member)
                response_member = MembersSchema(**member_data, actions=actions)
                response_members_list.append(response_member)

            return MembersResponse(corporation=response_members_list)

        @api.get(
            "owner/{owner_id}/view/paymentsystem/",
            response={200: PaymentSystemResponse, 403: dict, 404: dict},
            tags=self.tags,
        )
        def get_paymentsystem(request, owner_id: int):
            owner, perms = core.get_manage_owner(request, owner_id)

            if owner is None:
                return 404, {"error": _("Owner Not Found")}

            if perms is False:
                return 403, {"error": _("Permission Denied")}

            # Get Payment Accounts for Owner except those missing main character
            payment_accounts = (
                owner.payments_account_class.objects.filter(
                    owner=owner,
                    user__profile__main_character__isnull=False,
                )
                .exclude(status=owner.payments_account_class.Status.MISSING)
                .select_related(
                    "user", "user__profile", "user__profile__main_character"
                )
                .prefetch_related("user__character_ownerships__character")
            )

            # Use generic helper function
            payment_accounts_list = build_payment_accounts_response_list(
                payment_accounts, PaymentSystemSchema
            )

            return PaymentSystemResponse(owner=payment_accounts_list)

        @api.get(
            "corporation/admin/{corporation_id}/view/logs/",
            response={200: AdminLogResponse, 403: dict, 404: dict},
            tags=self.tags,
        )
        def get_corporation_admin_logs(request, corporation_id: int):
            owner, perms = core.get_manage_corporation(request, corporation_id)

            if owner is None:
                return 404, {"error": _("Corporation Not Found")}

            if perms is False:
                return 403, {"error": _("Permission Denied")}

            logs = (
                CorporationAdminHistory.objects.filter(owner=owner)
                .select_related("user")
                .order_by("-date")
            )

            # Use generic helper function
            response_admin_logs_list = build_admin_logs_response_list(
                logs, AdminHistorySchema
            )

            return AdminLogResponse(corporation=response_admin_logs_list)

        @api.get(
            "owner/{owner_id}/filter-set/{filter_set_id}/view/filter/",
            response={200: list[FilterModelSchema], 403: dict, 404: dict},
            tags=self.tags,
        )
        def get_filter_set_filters(request, owner_id: int, filter_set_id: int):
            owner, perms = core.get_manage_owner(request, owner_id)

            if owner is None:
                return 404, {"error": _("Corporation Not Found")}

            if perms is False:
                return 403, {"error": _("Permission Denied")}

            filters = owner.filter_class.objects.filter(
                filter_set__pk=filter_set_id,
            ).select_related("filter_set", "filter_set__owner")

            # Use generic helper function
            response_filter_list = build_filters_response_list(
                filters, FilterModelSchema
            )

            return render(
                request,
                "taxsystem/modals/view_filter.html",
                context={
                    "filters": response_filter_list,
                },
            )



# Standard Library
from unittest.mock import MagicMock, patch
from urllib import request

# Django
from django.test import override_settings
from django.utils import timezone

# Alliance Auth (External Libs)
from app_utils.testing import NoSocketsTestCase, create_user_from_evecharacter

# AA TaxSystem
from taxsystem.models.corporation import (
    CorporationFilter,
    CorporationPaymentAccount,
    CorporationPayments,
)
from taxsystem.tests.testdata.generate_filter import create_filter, create_filterset
from taxsystem.tests.testdata.generate_owneraudit import (
    create_corporation_owner_from_user,
)
from taxsystem.tests.testdata.generate_payments import (
    create_member,
    create_payment,
    create_payment_system,
)
from taxsystem.tests.testdata.load_allianceauth import load_allianceauth
from taxsystem.tests.testdata.load_eveuniverse import load_eveuniverse

MODULE_PATH = "taxsystem.managers.wallet_manager"


@override_settings(CELERY_ALWAYS_EAGER=True, CELERY_EAGER_PROPAGATES_EXCEPTIONS=True)
class TestPaymentsManager(NoSocketsTestCase):
    """Test Payments Manager for Corporation Journal Entries."""

    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        load_allianceauth()
        load_eveuniverse()

        cls.user, cls.character_ownership = create_user_from_evecharacter(
            1001,
        )

        cls.audit = create_corporation_owner_from_user(
            user=cls.user,
            tax_amount=1000,
            tax_period=30,
        )

        cls.payment_system = create_payment_system(
            name=cls.character_ownership.character.character_name,
            owner=cls.audit,
            user=cls.user,
            status=CorporationPaymentAccount.Status.ACTIVE,
            deposit=0,
            last_paid=(timezone.now() - timezone.timedelta(days=30)),
        )

        cls.payments = create_payment(
            name=cls.character_ownership.character.character_name,
            account=cls.payment_system,
            entry_id=1,
            amount=1000,
            date=timezone.datetime(2025, 1, 1, 12, 0, 0, tzinfo=timezone.utc),
            reason="Tax Payment",
            request_status=CorporationPayments.RequestStatus.PENDING,
            reviser="",
        )

        cls.payments2 = create_payment(
            name=cls.character_ownership.character.character_name,
            account=cls.payment_system,
            entry_id=2,
            amount=6000,
            date=timezone.datetime(2025, 1, 1, 14, 0, 0, tzinfo=timezone.utc),
            reason="Mining Stuff",
            request_status=CorporationPayments.RequestStatus.PENDING,
            reviser="",
        )

        cls.filter_set = create_filterset(
            owner=cls.audit,
            name="100m",
            description="Filter for payments over 100m",
        )

        cls.filter_amount = create_filter(
            filter_set=cls.filter_set,
            filter_type=CorporationFilter.FilterType.AMOUNT,
            value=1000,
        )

    def test_update_payments(self):
        # given

        self.audit.update_payment_system(force_refresh=False)

        self.assertSetEqual(
            set(
                self.payment_system.ts_corporation_payments.values_list(
                    "entry_id", flat=True
                )
            ),
            {1, 2},
        )
        obj = self.payment_system.ts_corporation_payments.get(entry_id=1)
        self.assertEqual(obj.amount, 1000)
        self.assertEqual(obj.request_status, CorporationPayments.RequestStatus.APPROVED)

        obj = self.payment_system.ts_corporation_payments.get(entry_id=2)
        self.assertEqual(obj.amount, 6000)
        self.assertEqual(
            obj.request_status, CorporationPayments.RequestStatus.NEEDS_APPROVAL
        )


@override_settings(CELERY_ALWAYS_EAGER=True, CELERY_EAGER_PROPAGATES_EXCEPTIONS=True)
class TestPaymentAccountManager(NoSocketsTestCase):
    """Test Payment Account Manager methods."""

    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        load_allianceauth()
        load_eveuniverse()

        cls.user, cls.character_ownership = create_user_from_evecharacter(1001)
        cls.user2, cls.character_ownership2 = create_user_from_evecharacter(1002)

        cls.audit = create_corporation_owner_from_user(
            user=cls.user,
            tax_amount=1000,
            tax_period=30,
        )

    def test_cleanup_orphaned_accounts(self):
        """Test that orphaned accounts are cleaned up."""
        # Create a payment account
        payment_account = create_payment_system(
            name=self.character_ownership.character.character_name,
            owner=self.audit,
            user=self.user,
            status=CorporationPaymentAccount.Status.ACTIVE,
        )

        # Remove the main character to make it orphaned
        self.user.profile.main_character = None
        self.user.profile.save()

        # Get empty auth accounts queryset
        auth_accounts = self.user.profile.__class__.objects.filter(
            main_character__isnull=False,
            main_character__corporation_id=self.audit.eve_corporation.corporation_id,
        )

        # Run cleanup
        CorporationPaymentAccount.objects._cleanup_orphaned_accounts(
            self.audit, auth_accounts
        )

        # Verify account was deleted
        self.assertFalse(
            CorporationPaymentAccount.objects.filter(pk=payment_account.pk).exists()
        )

    def test_process_accounts_creates_new_account(self):
        """Test that new payment accounts are created."""
        # Ensure no payment account exists
        CorporationPaymentAccount.objects.filter(user=self.user).delete()

        # Get auth accounts
        auth_accounts = self.user.profile.__class__.objects.filter(
            main_character__isnull=False,
            main_character__corporation_id=self.audit.eve_corporation.corporation_id,
        )

        # Process accounts
        new_accounts = CorporationPaymentAccount.objects._process_accounts(
            self.audit, auth_accounts
        )

        # Verify new account was created
        self.assertEqual(len(new_accounts), 1)
        self.assertEqual(new_accounts[0].user, self.user)
        self.assertEqual(
            new_accounts[0].status, CorporationPaymentAccount.Status.ACTIVE
        )

    def test_update_existing_account_owner_change(self):
        """Test that existing account is updated when owner changes."""
        # Create second corporation owner
        audit2 = create_corporation_owner_from_user(
            user=self.user2,
            tax_amount=2000,
            tax_period=30,
        )

        # Create payment account for first owner
        payment_account = create_payment_system(
            name=self.character_ownership.character.character_name,
            owner=audit2,
            user=self.user,
            status=CorporationPaymentAccount.Status.ACTIVE,
            deposit=500,
        )

        # Update to new owner
        CorporationPaymentAccount.objects._update_existing_account(
            self.audit,
            payment_account,
            self.character_ownership.character,
        )

        # Refresh from database
        payment_account.refresh_from_db()

        # Verify owner changed and deposit reset
        self.assertEqual(payment_account.owner, self.audit)
        self.assertEqual(payment_account.deposit, 0)

    def test_handle_corporation_change_mark_missing(self):
        """Test that account is marked as missing when user leaves corporation."""
        # Create payment account
        payment_account = create_payment_system(
            name=self.character_ownership.character.character_name,
            owner=self.audit,
            user=self.user,
            status=CorporationPaymentAccount.Status.ACTIVE,
        )

        # Simulate corporation change
        different_corp_id = 2001
        CorporationPaymentAccount.objects._handle_corporation_change(
            payment_account, different_corp_id
        )

        # Refresh from database
        payment_account.refresh_from_db()

        # Verify marked as missing
        self.assertEqual(
            payment_account.status, CorporationPaymentAccount.Status.MISSING
        )

    def test_reactivate_missing_account(self):
        """Test that missing account is reactivated correctly."""
        # Create missing payment account with data
        payment_account = create_payment_system(
            name=self.character_ownership.character.character_name,
            owner=self.audit,
            user=self.user,
            status=CorporationPaymentAccount.Status.MISSING,
            deposit=500,
            last_paid=timezone.now() - timezone.timedelta(days=10),
        )
        payment_account.notice = "User left corp"
        payment_account.save()

        # Reactivate
        CorporationPaymentAccount.objects._reactivate_missing_account(payment_account)

        # Refresh from database
        payment_account.refresh_from_db()

        # Verify reactivation
        self.assertEqual(
            payment_account.status, CorporationPaymentAccount.Status.ACTIVE
        )
        self.assertEqual(payment_account.deposit, 0)
        self.assertIsNone(payment_account.last_paid)
        self.assertIsNone(payment_account.notice)

    def test_check_payment_accounts_full_flow(self):
        """Test full payment account check flow."""
        # Ensure clean state
        CorporationPaymentAccount.objects.filter(owner=self.audit).delete()

        # Run check
        CorporationPaymentAccount.objects._check_payment_accounts(self.audit)

        # Verify new accounts were created (both user and user2 have main chars in same corp)
        payment_accounts = CorporationPaymentAccount.objects.filter(owner=self.audit)
        self.assertEqual(payment_accounts.count(), 2)

        # Verify first user account
        account1 = payment_accounts.get(user=self.user)
        self.assertEqual(account1.status, CorporationPaymentAccount.Status.ACTIVE)
        self.assertEqual(
            account1.name, self.character_ownership.character.character_name
        )

        # Verify second user account
        account2 = payment_accounts.get(user=self.user2)
        self.assertEqual(account2.status, CorporationPaymentAccount.Status.ACTIVE)
        self.assertEqual(
            account2.name, self.character_ownership2.character.character_name
        )

    def test_update_existing_account_reactivate_not_deactivated(self):
        """Test that inactive account is reactivated if not deactivated."""
        # Create inactive payment account
        payment_account = create_payment_system(
            name=self.character_ownership.character.character_name,
            owner=self.audit,
            user=self.user,
            status=CorporationPaymentAccount.Status.INACTIVE,
        )

        # Update account
        CorporationPaymentAccount.objects._update_existing_account(
            self.audit,
            payment_account,
            self.character_ownership.character,
        )

        # Refresh from database
        payment_account.refresh_from_db()

        # Verify reactivation
        self.assertEqual(
            payment_account.status, CorporationPaymentAccount.Status.ACTIVE
        )

    def test_update_existing_account_keep_deactivated(self):
        """Test that deactivated account stays deactivated."""
        # Create deactivated payment account
        payment_account = create_payment_system(
            name=self.character_ownership.character.character_name,
            owner=self.audit,
            user=self.user,
            status=CorporationPaymentAccount.Status.DEACTIVATED,
        )

        # Update account
        CorporationPaymentAccount.objects._update_existing_account(
            self.audit,
            payment_account,
            self.character_ownership.character,
        )

        # Refresh from database
        payment_account.refresh_from_db()

        # Verify stays deactivated
        self.assertEqual(
            payment_account.status, CorporationPaymentAccount.Status.DEACTIVATED
        )

    def test_handle_corporation_change_move_to_new_corp(self):
        """Test moving account to new corporation when user changes corps."""
        # Create second corporation
        audit2 = create_corporation_owner_from_user(
            user=self.user2,
            tax_amount=2000,
            tax_period=30,
        )

        # Create missing payment account for first corp
        payment_account = create_payment_system(
            name=self.character_ownership.character.character_name,
            owner=self.audit,
            user=self.user,
            status=CorporationPaymentAccount.Status.MISSING,
            deposit=500,
            last_paid=timezone.now(),
        )

        # Simulate moving to second corp
        CorporationPaymentAccount.objects._handle_corporation_change(
            payment_account,
            audit2.eve_corporation.corporation_id,
        )

        # Refresh from database
        payment_account.refresh_from_db()

        # Verify moved to new corp
        self.assertEqual(payment_account.owner, audit2)
        self.assertEqual(
            payment_account.status, CorporationPaymentAccount.Status.ACTIVE
        )
        self.assertEqual(payment_account.deposit, 0)
        self.assertIsNone(payment_account.last_paid)


@override_settings(CELERY_ALWAYS_EAGER=True, CELERY_EAGER_PROPAGATES_EXCEPTIONS=True)
class TestPayDayManager(NoSocketsTestCase):
    """Test Pay Day Manager methods."""

    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        load_allianceauth()
        load_eveuniverse()

        cls.user, cls.character_ownership = create_user_from_evecharacter(1001)

        cls.audit = create_corporation_owner_from_user(
            user=cls.user,
            tax_amount=1000,
            tax_period=30,
        )

    def test_pay_day_first_period_free(self):
        """Test that first period is free (last_paid is set but no deduction)."""
        # Create payment account without last_paid
        payment_account = create_payment_system(
            name=self.character_ownership.character.character_name,
            owner=self.audit,
            user=self.user,
            status=CorporationPaymentAccount.Status.ACTIVE,
            deposit=5000,
            last_paid=None,
        )

        # Run payday
        CorporationPaymentAccount.objects._pay_day(self.audit)

        # Refresh from database
        payment_account.refresh_from_db()

        # Verify first period is free
        self.assertIsNotNone(payment_account.last_paid)
        self.assertEqual(payment_account.deposit, 5000)  # No deduction

    def test_pay_day_deduct_tax(self):
        """Test that tax is deducted after period expires."""
        # Create payment account with expired last_paid
        old_date = timezone.now() - timezone.timedelta(days=31)
        payment_account = create_payment_system(
            name=self.character_ownership.character.character_name,
            owner=self.audit,
            user=self.user,
            status=CorporationPaymentAccount.Status.ACTIVE,
            deposit=5000,
            last_paid=old_date,
        )

        # Run payday
        CorporationPaymentAccount.objects._pay_day(self.audit)

        # Refresh from database
        payment_account.refresh_from_db()

        # Verify tax was deducted
        self.assertEqual(payment_account.deposit, 4000)  # 5000 - 1000
        self.assertNotEqual(payment_account.last_paid, old_date)

    def test_pay_day_skip_inactive(self):
        """Test that inactive accounts are skipped."""
        # Create inactive payment account
        payment_account = create_payment_system(
            name=self.character_ownership.character.character_name,
            owner=self.audit,
            user=self.user,
            status=CorporationPaymentAccount.Status.INACTIVE,
            deposit=5000,
            last_paid=timezone.now() - timezone.timedelta(days=31),
        )

        # Run payday
        CorporationPaymentAccount.objects._pay_day(self.audit)

        # Refresh from database
        payment_account.refresh_from_db()

        # Verify no deduction for inactive account
        self.assertEqual(payment_account.deposit, 5000)

    def test_pay_day_not_expired(self):
        """Test that tax is not deducted if period hasn't expired."""
        # Create payment account with recent last_paid
        recent_date = timezone.now() - timezone.timedelta(days=15)
        payment_account = create_payment_system(
            name=self.character_ownership.character.character_name,
            owner=self.audit,
            user=self.user,
            status=CorporationPaymentAccount.Status.ACTIVE,
            deposit=5000,
            last_paid=recent_date,
        )

        # Run payday
        CorporationPaymentAccount.objects._pay_day(self.audit)

        # Refresh from database
        payment_account.refresh_from_db()

        # Verify no deduction
        self.assertEqual(payment_account.deposit, 5000)
        self.assertEqual(payment_account.last_paid, recent_date)



# AA Tax System
# AA TaxSystem
from taxsystem.models.alliance import (
    AlliancePaymentAccount,
    AlliancePayments,
    AllianceUpdateStatus,
)
from taxsystem.models.corporation import (
    CorporationPaymentAccount,
    CorporationPayments,
    CorporationUpdateStatus,
    Members,
)


def create_payment(account: CorporationPaymentAccount, **kwargs) -> CorporationPayments:
    """Create a Payment for a Corporation"""
    params = {
        "account": account,
    }
    params.update(kwargs)
    payment = CorporationPayments(**params)
    payment.save()
    return payment


def create_member(owner: CorporationUpdateStatus, **kwargs) -> Members:
    """Create a Payment System for a Corporation"""
    params = {
        "owner": owner,
    }
    params.update(kwargs)
    member = Members(**params)
    member.save()
    return member


def create_payment_system(
    owner: CorporationUpdateStatus, **kwargs
) -> CorporationPaymentAccount:
    """Create a Payment System for a Corporation"""
    params = {
        "owner": owner,
    }
    params.update(kwargs)
    payment_system = CorporationPaymentAccount(**params)
    payment_system.save()
    return payment_system


def create_alliance_payment(
    account: AlliancePaymentAccount, **kwargs
) -> AlliancePayments:
    """Create a Payment for an Alliance"""
    params = {
        "account": account,
    }
    params.update(kwargs)
    payment = AlliancePayments(**params)
    payment.save()
    return payment


def create_alliance_payment_system(
    owner: AllianceUpdateStatus, **kwargs
) -> AlliancePaymentAccount:
    """Create a Payment System for an Alliance"""
    params = {
        "owner": owner,
    }
    params.update(kwargs)
    payment_system = AlliancePaymentAccount(**params)
    payment_system.save()
    return payment_system



# Django
from django.core.handlers.wsgi import WSGIRequest
from django.template.loader import render_to_string
from django.utils.html import format_html
from django.utils.safestring import mark_safe

# AA TaxSystem
from taxsystem.api.helpers import core
from taxsystem.models.alliance import AlliancePayments
from taxsystem.models.corporation import CorporationPayments, Members


def manage_payments(
    request: WSGIRequest, perms: str, payment: CorporationPayments | AlliancePayments
) -> str:
    """Generate the management action buttons for a payment"""
    # Create the action buttons
    actions = []
    if perms:
        if payment.is_pending or payment.is_needs_approval:
            actions.append(generate_payment_approve_button(payment=payment))
            actions.append(generate_payment_reject_button(payment=payment))
        elif payment.is_approved or payment.is_rejected:
            actions.append(generate_payment_undo_button(payment=payment))
        if payment.entry_id == 0:
            # Only allow deleting payments that are not ESI recorded
            actions.append(generate_payment_delete_button(payment=payment))
    # Get Payment Info Button
    if payment.account.user == request.user or perms:
        actions.append(core.generate_info_button(payment=payment))

    return format_html(
        f'<div class="d-flex justify-content-end">{format_html("".join(actions))}</div>'
    )


def generate_payment_approve_button(payment) -> mark_safe:
    """Generate a payment approve button for the tax system"""
    return format_html(
        render_to_string(
            "taxsystem/partials/buttons/payment-approve.html",
            {
                "payment": payment,
            },
        )
    )


def generate_payment_reject_button(payment) -> mark_safe:
    """Generate a payment reject button for the tax system"""
    return format_html(
        render_to_string(
            "taxsystem/partials/buttons/payment-reject.html",
            {
                "payment": payment,
            },
        )
    )


def generate_payment_undo_button(payment) -> mark_safe:
    """Generate a payment undo button for the tax system"""
    return format_html(
        render_to_string(
            "taxsystem/partials/buttons/payment-undo.html",
            {
                "payment": payment,
            },
        )
    )


def generate_payment_delete_button(payment) -> mark_safe:
    """Generate a payment delete button for the tax system"""
    return format_html(
        render_to_string(
            "taxsystem/partials/buttons/payment-delete.html",
            {
                "payment": payment,
            },
        )
    )


def generate_member_delete_button(member: Members) -> mark_safe:
    """Generate a member delete button for the tax system"""
    return format_html(
        render_to_string(
            "taxsystem/partials/buttons/member-delete.html",
            {
                "member": member,
            },
        )
    )


def generate_ps_toggle_button(account) -> mark_safe:
    """Generate a payment system toggle user button for the tax system"""
    return format_html(
        render_to_string(
            "taxsystem/partials/buttons/payment-system-toggle.html",
            {
                "account": account,
            },
        )
    )


def generate_ps_info_button(account) -> mark_safe:
    """Generate a payment system info button for the tax system"""
    return format_html(
        render_to_string(
            "taxsystem/partials/buttons/payment-system-info.html",
            {
                "account": account,
            },
        )
    )


def generate_filter_delete_button(filter_obj) -> mark_safe:
    """Generate a filter delete button for the tax system"""
    return format_html(
        render_to_string(
            "taxsystem/partials/buttons/filter-delete.html",
            {
                "filter": filter_obj,
            },
        )
    )


def generate_ps_add_payments_button(account) -> mark_safe:
    """Generate a payment system add payments button for the tax system"""
    return format_html(
        render_to_string(
            "taxsystem/partials/buttons/payment-add.html",
            {
                "account": account,
            },
        )
    )



# Django
from django.test import override_settings
from django.utils import timezone

# Alliance Auth (External Libs)
from app_utils.testing import NoSocketsTestCase, create_user_from_evecharacter

# AA TaxSystem
from taxsystem.models.alliance import AlliancePaymentAccount, AlliancePayments
from taxsystem.tests.testdata.generate_filter import create_filter, create_filterset
from taxsystem.tests.testdata.generate_owneraudit import (
    create_alliance_owner_from_user,
)
from taxsystem.tests.testdata.generate_payments import (
    create_alliance_payment,
    create_alliance_payment_system,
)
from taxsystem.tests.testdata.load_allianceauth import load_allianceauth
from taxsystem.tests.testdata.load_eveuniverse import load_eveuniverse


@override_settings(CELERY_ALWAYS_EAGER=True, CELERY_EAGER_PROPAGATES_EXCEPTIONS=True)
class TestAlliancePaymentAccountManager(NoSocketsTestCase):
    """Test Alliance Payment Account Manager methods."""

    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        load_allianceauth()
        load_eveuniverse()

        # Create users with alliance characters (1001 and 1004 both belong to alliance 3001)
        cls.user, cls.character_ownership = create_user_from_evecharacter(1001)
        cls.user2, cls.character_ownership2 = create_user_from_evecharacter(1004)

        cls.alliance = create_alliance_owner_from_user(
            user=cls.user,
            tax_amount=1000,
            tax_period=30,
        )

    def test_update_payment_accounts_creates_new_accounts(self):
        """Test that new payment accounts are created for alliance members."""
        # Ensure clean state
        AlliancePaymentAccount.objects.filter(owner=self.alliance).delete()

        # Run update
        AlliancePaymentAccount.objects._update_payment_accounts(self.alliance)

        # Verify accounts created for both users
        payment_accounts = AlliancePaymentAccount.objects.filter(owner=self.alliance)
        self.assertEqual(payment_accounts.count(), 2)

        # Verify first user account
        account1 = payment_accounts.get(user=self.user)
        self.assertEqual(account1.status, AlliancePaymentAccount.Status.ACTIVE)
        self.assertEqual(
            account1.name, self.character_ownership.character.character_name
        )

        # Verify second user account
        account2 = payment_accounts.get(user=self.user2)
        self.assertEqual(account2.status, AlliancePaymentAccount.Status.ACTIVE)

    def test_update_payment_accounts_moves_to_new_alliance(self):
        """Test that payment account is moved when user changes alliance."""
        # Create second alliance
        user3, _ = create_user_from_evecharacter(1003)
        alliance2 = create_alliance_owner_from_user(
            user=user3,
            tax_amount=2000,
            tax_period=30,
        )

        # Create payment account for first alliance
        payment_account = create_alliance_payment_system(
            name=self.character_ownership.character.character_name,
            owner=alliance2,
            user=self.user,
            status=AlliancePaymentAccount.Status.ACTIVE,
            deposit=500,
        )

        # Update to new alliance
        AlliancePaymentAccount.objects._update_payment_accounts(self.alliance)

        # Refresh from database
        payment_account.refresh_from_db()

        # Verify moved to new alliance and deposit reset
        self.assertEqual(payment_account.owner, self.alliance)
        self.assertEqual(payment_account.deposit, 0)

    def test_update_payment_accounts_reactivate_inactive(self):
        """Test that inactive accounts are reactivated."""
        # Create inactive payment account
        payment_account = create_alliance_payment_system(
            name=self.character_ownership.character.character_name,
            owner=self.alliance,
            user=self.user,
            status=AlliancePaymentAccount.Status.INACTIVE,
        )

        # Update accounts
        AlliancePaymentAccount.objects._update_payment_accounts(self.alliance)

        # Refresh from database
        payment_account.refresh_from_db()

        # Verify reactivation
        self.assertEqual(payment_account.status, AlliancePaymentAccount.Status.ACTIVE)

    def test_update_payment_accounts_keep_deactivated(self):
        """Test that deactivated accounts stay deactivated."""
        # Create deactivated payment account
        payment_account = create_alliance_payment_system(
            name=self.character_ownership.character.character_name,
            owner=self.alliance,
            user=self.user,
            status=AlliancePaymentAccount.Status.DEACTIVATED,
        )

        # Update accounts
        AlliancePaymentAccount.objects._update_payment_accounts(self.alliance)

        # Refresh from database
        payment_account.refresh_from_db()

        # Verify stays deactivated
        self.assertEqual(
            payment_account.status, AlliancePaymentAccount.Status.DEACTIVATED
        )

    def test_check_payment_accounts_mark_missing(self):
        """Test that accounts are marked missing when user leaves alliance."""
        # Create second alliance
        user3, char3 = create_user_from_evecharacter(1003)
        alliance2 = create_alliance_owner_from_user(
            user=user3,
            tax_amount=2000,
            tax_period=30,
        )

        # Create payment account for first alliance
        payment_account = create_alliance_payment_system(
            name=self.character_ownership.character.character_name,
            owner=self.alliance,
            user=self.user,
            status=AlliancePaymentAccount.Status.ACTIVE,
        )

        # Move user to second alliance by getting accounts from alliance2
        # This simulates the user being in a different alliance
        accounts = self.user.profile.__class__.objects.filter(
            main_character__isnull=False,
            main_character__alliance_id=alliance2.eve_alliance.alliance_id,
        )

        # Run check - since user is now in alliance2, their account in alliance1 should be marked missing
        # But we need to check with the user in a different alliance
        # Simulate by temporarily changing the user's alliance
        original_alliance_id = self.user.profile.main_character.alliance_id
        self.user.profile.main_character.alliance_id = (
            alliance2.eve_alliance.alliance_id
        )
        self.user.profile.main_character.save()

        # Get accounts with the updated alliance
        accounts = self.user.profile.__class__.objects.filter(
            main_character__isnull=False,
            main_character__alliance_id=alliance2.eve_alliance.alliance_id,
        )

        # Run check
        AlliancePaymentAccount.objects._check_payment_accounts(self.alliance, accounts)

        # Restore original alliance
        self.user.profile.main_character.alliance_id = original_alliance_id
        self.user.profile.main_character.save()

        # Refresh from database
        payment_account.refresh_from_db()

        # Verify marked as missing
        self.assertEqual(payment_account.status, AlliancePaymentAccount.Status.MISSING)

    def test_check_payment_accounts_reactivate_missing(self):
        """Test that missing accounts are reactivated when user returns."""
        # Create missing payment account with data
        payment_account = create_alliance_payment_system(
            name=self.character_ownership.character.character_name,
            owner=self.alliance,
            user=self.user,
            status=AlliancePaymentAccount.Status.MISSING,
            deposit=500,
            last_paid=timezone.now() - timezone.timedelta(days=10),
        )
        payment_account.notice = "User left alliance"
        payment_account.save()

        # Get accounts queryset (user is back in alliance)
        accounts = self.user.profile.__class__.objects.filter(
            main_character__isnull=False,
            main_character__alliance_id=self.alliance.eve_alliance.alliance_id,
        )

        # Run check
        AlliancePaymentAccount.objects._check_payment_accounts(self.alliance, accounts)

        # Refresh from database
        payment_account.refresh_from_db()

        # Verify reactivation
        self.assertEqual(payment_account.status, AlliancePaymentAccount.Status.ACTIVE)
        self.assertEqual(payment_account.deposit, 0)
        self.assertIsNone(payment_account.last_paid)
        self.assertIsNone(payment_account.notice)


@override_settings(CELERY_ALWAYS_EAGER=True, CELERY_EAGER_PROPAGATES_EXCEPTIONS=True)
class TestAlliancePayDayManager(NoSocketsTestCase):
    """Test Alliance Pay Day Manager methods."""

    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        load_allianceauth()
        load_eveuniverse()

        cls.user, cls.character_ownership = create_user_from_evecharacter(1001)

        cls.alliance = create_alliance_owner_from_user(
            user=cls.user,
            tax_amount=1000,
            tax_period=30,
        )

    def test_pay_day_first_period_free(self):
        """Test that first period is free (last_paid is set but no deduction)."""
        # Create payment account without last_paid
        payment_account = create_alliance_payment_system(
            name=self.character_ownership.character.character_name,
            owner=self.alliance,
            user=self.user,
            status=AlliancePaymentAccount.Status.ACTIVE,
            deposit=5000,
            last_paid=None,
        )

        # Run payday
        AlliancePaymentAccount.objects._pay_day(self.alliance)

        # Refresh from database
        payment_account.refresh_from_db()

        # Verify first period is free
        self.assertIsNotNone(payment_account.last_paid)
        self.assertEqual(payment_account.deposit, 5000)  # No deduction

    def test_pay_day_deduct_tax(self):
        """Test that tax is deducted after period expires."""
        # Create payment account with expired last_paid
        old_date = timezone.now() - timezone.timedelta(days=31)
        payment_account = create_alliance_payment_system(
            name=self.character_ownership.character.character_name,
            owner=self.alliance,
            user=self.user,
            status=AlliancePaymentAccount.Status.ACTIVE,
            deposit=5000,
            last_paid=old_date,
        )

        # Run payday
        AlliancePaymentAccount.objects._pay_day(self.alliance)

        # Refresh from database
        payment_account.refresh_from_db()

        # Verify tax was deducted
        self.assertEqual(payment_account.deposit, 4000)  # 5000 - 1000
        self.assertNotEqual(payment_account.last_paid, old_date)

    def test_pay_day_skip_inactive(self):
        """Test that inactive accounts are skipped."""
        # Create inactive payment account
        payment_account = create_alliance_payment_system(
            name=self.character_ownership.character.character_name,
            owner=self.alliance,
            user=self.user,
            status=AlliancePaymentAccount.Status.INACTIVE,
            deposit=5000,
            last_paid=timezone.now() - timezone.timedelta(days=31),
        )

        # Run payday
        AlliancePaymentAccount.objects._pay_day(self.alliance)

        # Refresh from database
        payment_account.refresh_from_db()

        # Verify no deduction for inactive account
        self.assertEqual(payment_account.deposit, 5000)

    def test_pay_day_not_expired(self):
        """Test that tax is not deducted if period hasn't expired."""
        # Create payment account with recent last_paid
        recent_date = timezone.now() - timezone.timedelta(days=15)
        payment_account = create_alliance_payment_system(
            name=self.character_ownership.character.character_name,
            owner=self.alliance,
            user=self.user,
            status=AlliancePaymentAccount.Status.ACTIVE,
            deposit=5000,
            last_paid=recent_date,
        )

        # Run payday
        AlliancePaymentAccount.objects._pay_day(self.alliance)

        # Refresh from database
        payment_account.refresh_from_db()

        # Verify no deduction
        self.assertEqual(payment_account.deposit, 5000)
        self.assertEqual(payment_account.last_paid, recent_date)



"""TestView class."""

# Standard Library
from http import HTTPStatus
from unittest.mock import Mock, patch

# Django
from django.contrib.sessions.middleware import SessionMiddleware
from django.test import RequestFactory, TestCase, override_settings
from django.urls import reverse

# Alliance Auth (External Libs)
from app_utils.testing import create_user_from_evecharacter

# AA TaxSystem
from taxsystem.models.corporation import CorporationOwner
from taxsystem.tests.testdata.load_allianceauth import load_allianceauth
from taxsystem.tests.testdata.load_eveuniverse import load_eveuniverse
from taxsystem.views import add_corp

MODULE_PATH = "taxsystem.views"


@patch(MODULE_PATH + ".messages")
@patch(MODULE_PATH + ".tasks")
@override_settings(CELERY_ALWAYS_EAGER=True, CELERY_EAGER_PROPAGATES_EXCEPTIONS=True)
class TestAddCorpView(TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        load_allianceauth()
        load_eveuniverse()

        cls.factory = RequestFactory()
        cls.user, cls.character_ownership = create_user_from_evecharacter(
            1001,
            permissions=[
                "taxsystem.basic_access",
                "taxsystem.create_access",
            ],
        )

    def _add_corporation(self, user, token):
        request = self.factory.get(reverse("taxsystem:add_corp"))
        request.user = user
        request.token = token
        middleware = SessionMiddleware(Mock())
        middleware.process_request(request)
        orig_view = add_corp.__wrapped__.__wrapped__.__wrapped__
        return orig_view(request, token)

    def test_add_corp(self, mock_tasks, mock_messages):
        # given
        user = self.user
        token = user.token_set.get(character_id=1001)
        # when
        response = self._add_corporation(user, token)
        # then
        self.assertEqual(response.status_code, HTTPStatus.FOUND)
        self.assertEqual(response.url, reverse("taxsystem:index"))
        self.assertTrue(mock_tasks.update_corporation.apply_async.called)
        self.assertTrue(mock_messages.info.called)
        self.assertTrue(
            CorporationOwner.objects.filter(
                eve_corporation__corporation_id=2001
            ).exists()
        )



# Django
from django.test import RequestFactory, TestCase
from django.utils import timezone

# Alliance Auth (External Libs)
from app_utils.testing import create_user_from_evecharacter

# AA TaxSystem
from taxsystem.api.alliance import PaymentAllianceSchema
from taxsystem.api.helpers.common import (
    build_own_payments_response_list,
    build_payments_response_list,
)
from taxsystem.models.alliance import AllianceOwner, AlliancePayments
from taxsystem.tests.testdata.generate_owneraudit import (
    create_alliance_owner_from_user,
    create_alliance_update_status,
)
from taxsystem.tests.testdata.generate_payments import (
    create_alliance_payment,
    create_alliance_payment_system,
)
from taxsystem.tests.testdata.load_allianceauth import load_allianceauth
from taxsystem.tests.testdata.load_eveuniverse import load_eveuniverse


class TestBuildAlliancePaymentsResponseList(TestCase):
    """
    Test build_payments_response_list helper function with Alliance data.

    Prerequisites:
    - load_allianceauth() and load_eveuniverse() must be called first
    - Test user must be created with an alliance character
    - At least one AllianceOwner must exist
    """

    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        # Load base test data (required for all tests)
        load_allianceauth()
        load_eveuniverse()

        cls.factory = RequestFactory()
        # Create user with alliance character (character 1001 belongs to alliance)
        cls.user, cls.character_ownership = create_user_from_evecharacter(
            1001, permissions=["taxsystem.basic_access"]
        )

        # Create AllianceOwner from user
        cls.alliance_owner = create_alliance_owner_from_user(cls.user)

        # Create AllianceUpdateStatus for payment system
        cls.update_status = create_alliance_update_status(
            owner_audit=cls.alliance_owner, section="payment_system"
        )

        # Create payment system account (owner is AllianceOwner, not UpdateStatus)
        cls.payment_account = create_alliance_payment_system(
            owner=cls.alliance_owner, user=cls.user, name="Test Alliance Account"
        )

    def test_builds_alliance_payment_list(self):
        """Test building alliance payment response list with AlliancePayments"""
        # Arrange
        payment1 = create_alliance_payment(
            account=self.payment_account,
            amount=1000000,
            date=timezone.now(),
            name="Test Payment 1",
            entry_id=1001,
            owner_id=self.alliance_owner.eve_alliance.alliance_id,
            reason="Test reason 1",
        )
        payment2 = create_alliance_payment(
            account=self.payment_account,
            amount=2000000,
            date=timezone.now(),
            name="Test Payment 2",
            entry_id=1002,
            owner_id=self.alliance_owner.eve_alliance.alliance_id,
            reason="Test reason 2",
        )

        payments = AlliancePayments.objects.filter(account=self.payment_account)
        request = self.factory.get("/")
        request.user = self.user
        perms = True

        # Act
        result = build_payments_response_list(
            payments, request, perms, PaymentAllianceSchema
        )

        # Assert
        self.assertEqual(len(result), 2)
        self.assertIsInstance(result, list)
        # Results are ordered by date descending, so payment2 comes first
        self.assertEqual(result[0].amount, payment2.amount)
        self.assertEqual(result[1].amount, payment1.amount)

    def test_builds_empty_list_for_no_alliance_payments(self):
        """Test building empty list when no alliance payments exist"""
        # Arrange
        payments = AlliancePayments.objects.none()
        request = self.factory.get("/")
        request.user = self.user
        perms = True

        # Act
        result = build_payments_response_list(
            payments, request, perms, PaymentAllianceSchema
        )

        # Assert
        self.assertEqual(len(result), 0)
        self.assertIsInstance(result, list)


class TestBuildOwnAlliancePaymentsResponseList(TestCase):
    """
    Test build_own_payments_response_list helper function with Alliance data.

    Prerequisites:
    - load_allianceauth() and load_eveuniverse() must be called first
    - Test user must be created with an alliance character
    - At least one AllianceOwner must exist
    """

    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        # Load base test data (required for all tests)
        load_allianceauth()
        load_eveuniverse()

        # Create user with alliance character
        cls.user, cls.character_ownership = create_user_from_evecharacter(
            1001, permissions=["taxsystem.basic_access"]
        )

        # Create AllianceOwner from user
        cls.alliance_owner = create_alliance_owner_from_user(cls.user)

        # Create AllianceUpdateStatus for payment system
        cls.update_status = create_alliance_update_status(
            owner_audit=cls.alliance_owner, section="payment_system"
        )

        # Create payment system account (owner is AllianceOwner, not UpdateStatus)
        cls.payment_account = create_alliance_payment_system(
            owner=cls.alliance_owner, user=cls.user, name="Test Alliance Account"
        )

    def test_builds_own_alliance_payment_list(self):
        """Test building own payment response list for alliance"""
        # Arrange
        payment1 = create_alliance_payment(
            account=self.payment_account,
            amount=500000,
            date=timezone.now(),
            name="Own Payment 1",
            entry_id=2001,
            owner_id=self.alliance_owner.eve_alliance.alliance_id,
            reason="Own reason 1",
        )
        payment2 = create_alliance_payment(
            account=self.payment_account,
            amount=750000,
            date=timezone.now(),
            name="Own Payment 2",
            entry_id=2002,
            owner_id=self.alliance_owner.eve_alliance.alliance_id,
            reason="Own reason 2",
        )

        payments = AlliancePayments.objects.filter(account=self.payment_account)

        # Act
        result = build_own_payments_response_list(payments, PaymentAllianceSchema)

        # Assert
        self.assertEqual(len(result), 2)
        self.assertIsInstance(result, list)
        # Results are ordered by date descending, so payment2 comes first
        self.assertEqual(result[0].amount, payment2.amount)
        self.assertEqual(result[1].amount, payment1.amount)

    def test_builds_empty_own_alliance_payment_list(self):
        """Test building empty own alliance payment list"""
        # Arrange
        payments = AlliancePayments.objects.none()

        # Act
        result = build_own_payments_response_list(payments, PaymentAllianceSchema)

        # Assert
        self.assertEqual(len(result), 0)
        self.assertIsInstance(result, list)



"""General Model"""

# Standard Library
from dataclasses import dataclass
from typing import Any, NamedTuple

# Django
from django.db import models
from django.utils.translation import gettext_lazy as _


class General(models.Model):
    """General model for app permissions"""

    class Meta:
        managed = False
        permissions = (
            ("basic_access", _("Can access the Tax System")),
            ("create_access", _("Can add Corporation")),
            ("manage_own_corp", _("Can manage own Corporation")),
            ("manage_corps", _("Can manage all Corporations")),
            ("manage_own_alliance", _("Can manage own Alliance")),
            ("manage_alliances", _("Can manage all Alliances")),
        )
        default_permissions = ()


class UpdateSection(models.TextChoices):
    """Base class for update sections."""

    @classmethod
    def get_sections(cls) -> list[str]:
        """Return list of section values."""
        return [choice.value for choice in cls]

    @property
    def method_name(self) -> str:
        """Return method name for this section."""
        return f"update_{self.value}"


class CorporationUpdateSection(UpdateSection):
    """Sections for corporation updates."""

    DIVISION_NAMES = "division_names", _("Wallet Division Names")
    DIVISION = "division", _("Wallet Division")
    WALLET = "wallet", _("Wallet Journal")
    MEMBERS = "members", _("Members")
    PAYMENT_SYSTEM = "payment_system", _("Payment System")
    PAYMENTS = "payments", _("Payments")
    PAYDAY = "payday", _("Payday")


class AllianceUpdateSection(UpdateSection):
    """Sections for alliance updates."""

    PAYMENT_SYSTEM = "payment_system", _("Payment System")
    PAYMENTS = "payments", _("Payments")
    PAYDAY = "payday", _("Payday")


class UpdateSectionResult(NamedTuple):
    """A result of an attempted section update."""

    is_changed: bool | None
    is_updated: bool
    has_token_error: bool = False
    error_message: str | None = None
    data: Any = None


@dataclass(frozen=True)
class _NeedsUpdate:
    """An Object to track if an update is needed."""

    section_map: dict[str, bool]

    def __bool__(self) -> bool:
        """Check if any section needs an update."""
        return any(self.section_map.values())

    def for_section(self, section: str) -> bool:
        """Check if an update is needed for a specific section."""
        return self.section_map.get(section, False)
